//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: AppOption.h,v 1.17 2001/03/19 05:33:40 tujikawa Exp $

#ifndef _APPOPTION_H_
#define _APPOPTION_H_
#include <iostream.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <dirent.h>
#include "aria.h"
#include "ThreadManager.h"
#include "ItemManager.h"
#include "Dialog.h"
#include "CommandList.h"
#include "ItemList.h"
#include "TimerData.h"

#define DEFAULT_ICONSET "<DEFAULT>"

class AppOption {
private:
  pthread_mutex_t option_lock;
  unsigned int maxthread;
  bool use_automatic_start;
  GtkWidget *notebook;
  GtkWidget *option_window;
  GtkWidget *spin_thread;
  GtkWidget *toggle_automatic_start;
  bool use_ignore_error_item;
  GtkWidget *ignore_error_item_toggle;
  GtkWidget *ignore_extention_toggle;
  GtkWidget *ignore_extention_entry;
  list<string> ignore_extention_list;
  bool use_ignore_extention_list;
  GtkWidget *conform_delete_toggle;
  GtkWidget *conform_exit_toggle;
  bool conform_delete;
  bool conform_exit;
  GtkWidget *use_servertemplate_toggle;
  bool use_servertemplate;
  GtkWidget *svt_clist;

  GtkWidget *use_commandlist_toggle;
  bool use_commandlist;
  GtkWidget *com_clist;

  bool use_size_human_readable;
  GtkWidget *use_size_human_readable_toggle;

  // icon set
  string icon_dir;
  GtkWidget *icon_dir_clist;
  GtkWidget *icon_preview_clist;

  // autosave
  bool use_autosave;
  unsigned int autosave_interval;
  int autosave_tag;
  GtkWidget *autosave_spin;
  GtkWidget *autosave_toggle;

  // run command when all downloads are over
  bool use_arb_command;
  string arb_command;
  GtkWidget *arb_command_entry;
  GtkWidget *use_arb_command_toggle;

  // simple timer
  bool use_timer;
  int timer_start_tag;
  int timer_stop_tag;
  TimerData timerdata;
  GtkWidget *timer_hour_start_spin;
  GtkWidget *timer_min_start_spin;
  GtkWidget *timer_hour_stop_spin;
  GtkWidget *timer_min_stop_spin;
  GtkWidget *use_timer_toggle;
  GtkWidget *iterate_timer_toggle;

public:
  AppOption(GtkWidget *app_window);

  void Set_Option_Values(unsigned int maxthread_in,
			 bool use_automatic_start,
			 bool use_ignore_error_item_in,
			 bool use_ignore_extention_list,
			 const list<string>& ignore_extention_list_in,
			 bool use_autosave,
			 unsigned int autosave_interval,
			 bool use_arb_command,
			 const string& arb_command,
			 bool use_timer,
			 const TimerData& timerdata,
			 bool conform_delete,
			 bool conform_exit,
			 bool use_servertemplate,
			 bool use_commandlist,
			 bool use_size_human_readable,
			 const string& icon_dir,
			 const list<string>& svt_names,
			 const list<string>& command_names);
  void       Show_option_window();
  void Set_icon_dir();
  GtkWidget *ret_Option_Window() const;
  void       Process_changes();
  GtkWidget *Create_COMMANDLIST_page();
  GtkWidget *Create_SERVERTEMPLATE_page();
  GtkWidget *Create_DOWNLOAD_page();
  GtkWidget *Create_CONFORMATION_page();
  GtkWidget *Create_DISPLAY_page();
  GtkWidget *Create_TIMER_page();

  unsigned int      ret_Maxthread() const;
  void       set_Maxthread(unsigned int maxthread_in);
  bool       Increase_thread();
  bool       Decrease_thread();
  bool       Whether_use_automatic_start();
  bool       ret_use_ignore_error_item();
  bool       Is_in_ignore_extention_list(const string& file);
  const list<string>& ret_ignore_extention_list();
  bool       ret_use_ignore_extention_list();
  bool       ret_conform_delete();
  bool       ret_conform_exit();
  bool       ret_use_servertemplate();
  bool       ret_use_commandlist();
  bool       ret_use_size_human_readable();
  bool       ret_use_autosave();
  unsigned int ret_autosave_interval();
  int        ret_autosave_tag();
  list<string> ret_name_list(GtkWidget *clist);
  list<string> ret_svt_name_list();
  list<string> ret_com_name_list();
  void set_svt_clist(const list<string>& names);
  void set_com_clist(const list<string>& names);
  
  void set_icon_dir(const string& icon_dir);
  const string& ret_icon_dir();
  bool ret_use_arb_command();
  string ret_arb_command();
  bool ret_use_timer();
  bool ret_iterate_timer();
  time_t ret_timer_start_time();
  time_t ret_timer_stop_time();
  void Start_start_timer(unsigned int check_interval);
  void Start_stop_timer(unsigned int check_interval);
  void Update_timer_start();
  void Update_timer_stop();
  int ret_timer_hour_start();
  int ret_timer_min_start();
  int ret_timer_hour_stop();
  int ret_timer_min_stop();
};

#endif // _APPOPTION_H_
