//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemCell_FTP_p.cc,v 1.16 2001/03/19 05:33:40 tujikawa Exp $

// class ItemCell_ftp implementation

#include "ItemCell_FTP_p.h"

ItemCell_FTP_p::ItemCell_FTP_p(const string& url_in,
			       const URLcontainer& urlcon_in,
			       const Options& options_in,
			       const string& initial_log,
			       ItemCell* itemcell_boss,
			       unsigned int order_in,
			       unsigned int start_range_in,
			       unsigned int end_range_in)
  :ItemCell_FTP(url_in, urlcon_in, options_in, initial_log),
   ItemCell_Partial(url_in, urlcon_in, options_in, initial_log, itemcell_boss, order_in, start_range_in, end_range_in),
   ItemCell(url_in, urlcon_in, options_in, initial_log)
{
}

ItemCell_FTP_p::~ItemCell_FTP_p()
{
}

ItemCell::DownloadStatusType ItemCell_FTP_p::Download_Main()
{
  unsigned int startingbyte = 0;
  string report;
  string command;
  string username;
  string password;
  string retbuf;

  try {
    // connection established
    if(ret_Status() != ITEM_CRCERROR &&
       ret_Status() != ITEM_EXECERROR &&
       ret_Status() != ITEM_DOWNLOAD_AGAIN &&
       ret_Options().ret_downm_type() != Options::DOWNM_NORESUME) {
      startingbyte = Get_starting_byte();
    }
    set_Status(ITEM_DOWNLOAD);
    Send_status();
    
    // connect to ftp server    
    Socket sock_command;
    if(sock_command.bad()) {
      throw ITEM_EIO;
    }
    Make_TCP_connection(sock_command,ret_URL_Container().ret_Port());

    if(Get_response(sock_command, retbuf) < 0) {
      set_Command(DLERROR);
      throw ITEM_EPROT;
    }

    // authentication phase
    if(ret_Options().Whether_use_authentication() &&
       ret_Options().ret_User() != "") {
      username = ret_Options().ret_User();
      password = ret_Options().ret_Password();
    } else {
      username = "anonymous";
      password = "IE40user@";
    }
    // send USER
    command = "USER "+username+"\r\n";
    Send_command(command, sock_command);
    if(Get_response(sock_command, retbuf) < 0) {
      set_Command(DLERROR);
      throw ITEM_EPROT;
    }

    // send PASS
    command = "PASS "+password+"\r\n";
    Send_command_pass(command, sock_command);
    
    if(Get_response(sock_command, retbuf) < 0) {
      Send_message_to_gui(_("faild to login"), MSG_DOWNLOAD_ERROR);
      set_Command(DLERROR);
      throw ITEM_EPROT;
    }
    
    // send TYPE command
    switch(ret_Options().ret_FTP_ret_mode()) {
    case Options::FTP_BINARY:
      Send_command("TYPE I\r\n", sock_command);
      break;
    case Options::FTP_ASCII:
    default:
      Send_command("TYPE A\r\n", sock_command);
      break;
    }
    if(Get_response(sock_command, retbuf) < 0) {
      set_Command(DLERROR);
      throw ITEM_EPROT;
    }
    // send CWD command
    //if(ret_Options().Whether_use_authentication() &&
    // ret_Options().ret_User() != "") {
    // command = "CWD ."+ret_URL_Container().ret_Dir()+"\r\n";
    //} else {
      command = "CWD "+ret_URL_Container().ret_Dir()+"\r\n";
      //}
    Send_command(command, sock_command);
    
    if(Get_response(sock_command, retbuf) < 0) {
      Send_message_to_gui(_("specified directory is not found"), MSG_DOWNLOAD_ERROR);
      set_Command(DLERRORSTOP);
      throw ITEM_EPROT;
    }
    
    if(startingbyte > 0 &&
       ret_Options().ret_downm_type() == Options::DOWNM_IFMODSINCE) {
      command = "MDTM "+ret_URL_Container().ret_Filename()+"\r\n";
      Send_command(command, sock_command);
      if(Get_response(sock_command, retbuf) < 0) {
	Send_message_to_gui(_("modtime command failed. anyway, try resume"), MSG_DOWNLOAD_INFO);
      } else {
	  time_t modtime = get_mod_time(retbuf);
	  if(Is_older_than_remote(modtime)) {
	    Send_message_to_gui(_("modification time of remote file is newer than that of local one. resume disabled"), MSG_DOWNLOAD_INFO);
	    startingbyte = 0;
	  }
      }
    }
    // we already know the size of partial file
    set_Size_Total(end_range-start_range);
    
    if(ret_Size_Total() == startingbyte) {
      //Ǥ˥ɺ
      // downloading has been competed
      report = "'"+ret_Filename()+"'"+_(" is already downloaded");
      Send_message_to_gui(report, MSG_DOWNLOAD_INFO);
      
      set_Size_Current(ret_Size_Total());
      Send_status();
      Leave_ftp_server(sock_command);
      Send_message_to_gui(_("connection closed"), MSG_DOWNLOAD_INFO);
      throw ITEM_ESUCCESS;
    } else if(ret_Size_Total() < startingbyte) {
      // ե륵ۤʤ
      // size mismatch
      Send_message_to_gui(_("size of local file is larger than that of remote file. Download again"), MSG_DOWNLOAD_ERROR);
      startingbyte = 0;
      set_Size_Total(end_range-start_range);
    } else {
      set_Size_Total(end_range-start_range);
    }

    Options::FTP_Mode ftp_mode = ret_Options().ret_FTP_Mode();
    
    switch(ftp_mode) {
    case Options::FTP_PASSIVE_MODE: // PASV mode
      {
	unsigned int startingbyte_real = startingbyte+start_range;
	Socket sock_data;
	if(sock_data.bad()) {
	  throw ITEM_EIO;
	}
	Handle_ftp_passive_mode(sock_command, sock_data, startingbyte_real);
	Start_Download(sock_data, startingbyte);
	sock_data.Shutdown(2);
	break;
      }
    case Options::FTP_ACTIVE_MODE:
    default: //ACTIVE(NORMAL) mode
      {
	Socket sock_wait;
	if(sock_wait.bad()) {
	  throw ITEM_EIO;
	}
	unsigned int startingbyte_real = startingbyte+start_range;
	Socket sock_data = Handle_ftp_active_mode(sock_command, startingbyte_real);
	Start_Download(sock_data, startingbyte);
	sock_data.Shutdown(2);
	break;
      }
    }
    // ftp server will return negative response:
    // "426 Connection reset by peer."
    // this message can be ignored safely
    try {
      Get_response(sock_command, retbuf);
      
      if(!ret_Options().ret_FTP_nosend_quit()) {
	Send_command("QUIT\r\n", sock_command);
	// expected reply: "221 Goodbye." or something like this
	Get_response(sock_command, retbuf);
      }
    } catch (ItemErrorType err) {
      PERROR(err);
      Send_message_to_gui(_("an error occured, but this error can be ignored safely"), MSG_DOWNLOAD_INFO);
      throw ITEM_ESUCCESS;
    }

    throw ITEM_ESUCCESS;
  }
  catch(ItemErrorType err) {
    switch(err) {
    case ITEM_ESUCCESS:
      set_Command(DLSUCCESS);
      return DLSUCCESS;
    case ITEM_EINTER:
      return ret_Command();
    case ITEM_EIOFILE:
      set_Command(DLERRORSTOP);
      PERROR(err);
      return DLERRORSTOP;
    case ITEM_EPROT:// modified
      PERROR(err);
      return ret_Dl_status();
    default:
      PERROR(err);
      set_Command(DLERROR);
      return DLERROR;
    }
  }
}

//
// ե
//
// return:
//     -1: 顼
//      0: 桼ˤ
//      1: 
//
void ItemCell_FTP_p::Start_Download(const Socket& socket, unsigned int startingbyte)
{
  ofstream outfile;
  string line;

  Make_directory_if_needed();

  try {
    line = _("a part of '")+ret_URL()+"'"+_(", its index is ")+itos(order)+_(", its range is from ")+itos(start_range, true)+" to "+itos(end_range, true);
    Send_message_to_gui(line, MSG_DOWNLOAD_INFO);

    line = _("starting download at ")+itos(start_range+startingbyte, true)+_("bytes");
    Send_message_to_gui(line, MSG_DOWNLOAD_INFO);

    string filename =  ret_Options().ret_Store_Dir()+ret_Filename();

    switch(SplitNumberChanged(filename+".index")) {
    case PARTIAL_NORMAL:
      break;
    case PARTIAL_CHANGED:
      //Send_message_to_gui(_("split count has changed"), MSG_DOWNLOAD_INFO);
    default:
      startingbyte = 0;
    }

    if(startingbyte == 0) {
      // ˥
      outfile.open(filename.c_str(), ios::out|ios::trunc|ios::binary);
    } else {
      // 쥸塼ह
      outfile.open(filename.c_str(), ios::out|ios::app|ios::binary);
    }
    if(outfile.bad()) {
      throw ITEM_EIOFILE;
    }

    set_Size_Current(startingbyte);
    Download_data(outfile, socket);
    outfile.close();
  } catch (ItemErrorType err) {
    throw err;
  }
}
