//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemOption.h,v 1.23 2001/03/22 08:42:18 tujikawa Exp $

#ifndef _ITEMOPTION_H_
#define _ITEMOPTION_H_

#include <gtk/gtk.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <algorithm>
#include "aria.h"
#include "ProxyList.h"
#include "UseragentList.h"
#include "ItemManager.h"
#include "utils.h"
#include "ItemCell.h"
#include "Dialog.h"
#include "PasteWindow.h"
using namespace std;

#define DEFAULT_PROXY_PORT 8080

enum OptionErrorType {
  OPTION_INVALID_DIR,
  OPTION_INVALID_URL,
  OPTION_INVALID_CRC,
  OPTION_SUCCESS
};

class ItemOption {
 private:
  ItemCell *itemcell;

  // option window
  GtkWidget *option_window;
  GtkWidget *notebook;
  // URL page
  GtkWidget *url_entry, *crc_entry;
  GtkWidget *toggle_use_authentication;
  GtkWidget *user_entry, *password_entry;
  GtkWidget *storedir_entry;
  // HTTP1 page
  GtkWidget *referer_entry, *timedout_entry;
  GtkWidget *none_button, *index_button, *user_button, *urlref_button;
  GtkWidget *toggle_use_http_proxy_authentication;
  GtkWidget *http_proxy_user_entry, *http_proxy_password_entry;
  GtkWidget *http_proxy_cbox;
  GtkWidget *toggle_use_http_cache;
  GtkWidget *http_proxy_add, *http_proxy_delete;
  GtkWidget *use_http_proxy_button, *not_use_http_proxy_button;
  GtkWidget *use_retry_404_toggle;
  GtkWidget *use_retry_503_toggle;
  GtkWidget *http_version_select;
  // HTTP2 page
  GtkWidget *useragent_cbox;
  GtkWidget *toggle_random_useragent;
  GtkWidget *cookie_delete_on_restart_toggle;
  GtkWidget *cookie_nosend_toggle;
  GtkWidget *useifmodsince_toggle;
  GtkWidget *noresume_toggle;
  GtkWidget *alwaysresume_toggle;
  // FTP page
  GtkWidget *active_button, *passive_button;
  GtkWidget *ftp_nosend_quit_toggle;
  GtkWidget *ftp_recurse_depth_spin;
  GtkWidget *ftp_no_ascend_toggle;
  GtkWidget *ftp_get_symlink_as_realfile_toggle;
  GtkWidget *ftp_use_filter_toggle;
  GtkWidget *ftp_allow_crawl_toggle;
  GtkWidget *ftp_filter_entry;
  GtkWidget *ftp_filter_clist;

  GtkWidget *binary_button, *ascii_button;
  // DOWNLOAD1 page
  GtkWidget *toggle_delete_when_finish;
  GtkWidget *toggle_dont_delete_without_crc;
  GtkWidget *no_crc_check_toggle;
  GtkWidget *ignore_crc_error_toggle;
  GtkWidget *spin_timedout, *spin_divide;
  GtkWidget *spin_retry, *spin_retry_push_back;
  GtkWidget *rollback_bytes_entry;
  GtkWidget *speed_scale;
  // HTTP3 page
  GtkWidget *documentroot_dir_entry;
  GtkWidget *recurse_depth_spin;

  GtkWidget *recurse_hostname_dir_toggle;
  GtkWidget *recurse_abs2rel_toggle;
  GtkWidget *recurse_force_convert_toggle;
  GtkWidget *recurse_del_comment_toggle;
  GtkWidget *recurse_del_javascript_toggle;
  GtkWidget *recurse_no_other_host_toggle;
  GtkWidget *recurse_no_ascend_toggle;
  GtkWidget *recurse_relative_only_toggle;
  GtkWidget *recurse_referer_override_toggle;
  GtkWidget *recurse_follow_ftp_link_toggle;
  GtkWidget *recurse_convert_tilde_toggle;

  GtkWidget *recurse_tag_href_toggle;
  GtkWidget *recurse_tag_src_toggle;
  GtkWidget *recurse_tag_background_toggle;
  GtkWidget *recurse_tag_code_toggle;

  GtkWidget *recurse_parse_target_entry;
  GtkWidget *recurse_nodown_filter_target_entry;
  GtkWidget *recurse_parse_target_clist;
  GtkWidget *recurse_nodown_filter_target_clist;
  GtkWidget *recurse_ign_domain_entry;
  GtkWidget *recurse_ign_domain_clist;

  GtkWidget *recurse_down_filter_target_entry;
  GtkWidget *recurse_down_filter_target_clist;
  GtkWidget *recurse_use_down_filter_toggle;

public:
  ItemOption(GtkWidget *app_window);//constructor

  virtual void set_Option_Values(ItemCell *itemcell, const Options& options_temp);
  virtual void Show_Option_Window();

  int Process_changes();
  GtkWidget *ret_Option_Window() const;
  ItemCell *ret_ItemCell() const;

  // create option window
  GtkWidget *Create_URL_page();
  GtkWidget *Create_HTTP1_page();
  GtkWidget *Create_HTTP2_page();
  GtkWidget *Create_Download1_page();
  GtkWidget *Create_HTTP3_page();
  GtkWidget *Create_FTP_page();
  void set_http_proxy_list(const ProxyList *http_proxy_list);
  void set_useragent_list(const UseragentList *useragent_list);
  void set_Store_Dir(const char *dir);
};

#endif // _ITEMOPTION_H_
