//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemStatusDynamic.cc,v 1.10 2001/03/19 05:33:41 tujikawa Exp $

#include "ItemStatusDynamic.h"

extern bool Download_delete_sub(ItemCell *itemcell);
extern void Set_clist_column__crc(int rowindex, const string& crc_string);
extern void Set_clist_column__cursize(int rowindex, const string& size_string);
extern void Set_clist_column__totsize(int rowindex, const string& size_string);
extern void Set_clist_column__progress(int rowindex, int progress);
extern void Set_clist_column__filename(int rowindex, const string& filename);
extern void Set_clist_column__speed(int rowindex, const string& speed_string);
extern void Set_clist_column__rtime(int rowindex, const string& rtime_string);
extern void Set_clist_column__status(int rowindex, const string& status_string);
extern void Set_clist_column__rec(int rowindex, const string& rec_string);
extern void Set_clist_column__url(int rowindex, const string& url_string);
extern void Set_clist_column__save(int rowindex, const string& save_string);
extern void Set_clist_column__try(int rowindex, const string& try_string);

extern void Set_clist_column_style__download(int rowindex);
extern void Set_clist_column_style__stop(int rowindex);
extern void Set_clist_column_style__error(int rowindex);
extern void Set_clist_column_style__complete(int rowindex);
extern void Set_clist_column_style__ready(int rowindex);
extern void Set_clist_column_style__crcerror(int rowindex);
extern void Set_clist_column_style__divided(int rowindex);
extern void Set_clist_column_style__concat(int rowindex);
extern void Set_clist_column__icon(int rowindex , ItemCell::ItemStatusType status);
extern void Set_sensitive__no_item_selected();
extern void Set_sensitive__list_empty();
extern bool Is_track_required();
extern ItemManager *item_manager;
extern pthread_mutex_t itemlistlock;
extern GtkWidget *itemlistwidget;
extern AppOption *appoption;

ItemStatusDynamic::ItemStatusDynamic(ItemCell *itemcell_in, ItemCell::ItemStatusType status_in, unsigned int retry_count_in, unsigned int size_current_in, unsigned int size_total_in) : ItemStatus(itemcell_in)
{
  size_current = size_current_in;
  size_total = size_total_in;
  retry_count = retry_count_in;
  status = status_in;
  speed = 0.0;
  delete_flag = false;
  update_flag = name_col | progress_col | crc_col | static_col;
}

ItemStatusDynamic::ItemStatusDynamic(ItemCell *itemcell_in, ItemCell::ItemStatusType status_in, unsigned int retry_count_in, unsigned int size_current_in, unsigned int size_total_in, float speed_in) : ItemStatus(itemcell_in)
{
  size_current = size_current_in;
  size_total = size_total_in;
  retry_count = retry_count_in;
  status = status_in;
  speed = speed_in;
  delete_flag = false;
  update_flag = speed_col | progress_col;
}

ItemStatusDynamic::~ItemStatusDynamic()
{
}

void ItemStatusDynamic::Update()
{
  ItemCell *itemcell = ret_ItemCell();
  if(!item_manager->search_item(itemcell)) {
    return;
  }
  //pthread_mutex_lock(&itemlistlock);
  //gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  int rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
  if(update_flag & ItemStatusDynamic::name_col) {
    if(itemcell->Is_Partial()) {
      string line = "|->"+itemcell->ret_Filename();
      Set_clist_column__filename(rowindex, line);
    } else {
      Set_clist_column__filename(rowindex, itemcell->ret_Filename());
    }
  }
  if(update_flag & ItemStatusDynamic::progress_col) {
    int progress = 0;
    if(size_total != 0) {
      progress = (int)(100.0*((float)size_current/size_total));//fixed 2001/3/18
    }
    Set_clist_column__progress(rowindex, progress);
    if(appoption->ret_use_size_human_readable()) {
      Set_clist_column__cursize(rowindex, get_human_readable_size(size_current));
      Set_clist_column__totsize(rowindex, get_human_readable_size(size_total));
    } else {
      Set_clist_column__cursize(rowindex, itos(size_current, true));
      Set_clist_column__totsize(rowindex, itos(size_total, true));
    }
  }
  if(update_flag & ItemStatusDynamic::speed_col) {
    string speed_string = ftos(speed, 1)+"k";
    Set_clist_column__speed(rowindex, speed_string.c_str());

    int rtime = (int)((size_total-size_current)/speed/1024);
    if(rtime < 0 || size_total == 0) {
      Set_clist_column__rtime(rowindex, "--:--:--");
    } else {
      int hour = rtime/3600;
      int temp = rtime % 3600;
      int min = temp/60;
      int sec = temp % 60;
      Set_clist_column__rtime(rowindex, itos(hour, 2, '0')+":"+itos(min, 2, '0')+":"+itos(sec, 2, '0'));
    }
  }
  if(update_flag & ItemStatusDynamic::static_col) {
    Set_clist_column__try(rowindex, itos(retry_count)+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));
    Set_clist_column__save(rowindex, itemcell->ret_Options().ret_Store_Dir().c_str());
    Set_clist_column__url(rowindex, itemcell->ret_URL().c_str());
    if(itemcell->ret_URL_Container().ret_Protocol() == "http:") {
      Set_clist_column__rec(rowindex, itos(itemcell->ret_Options().ret_recurse_count()));
    } else {
      Set_clist_column__rec(rowindex, itos(itemcell->ret_Options().ret_FTP_recurse_count()));
    }
  }
  Set_clist_column__icon(rowindex, status);

  switch(status) {
  case ItemCell::ITEM_READY:
  case ItemCell::ITEM_READY_AGAIN:
    //Set_clist_column_style__ready(rowindex);
    //Set_clist_column__status(rowindex, _("READY"));
    break;
  case ItemCell::ITEM_READY_CONCAT:
    //Set_clist_column_style__concat(rowindex);
    //Set_clist_column__status(rowindex, _("READY TO CONCAT"));
    break;
  case ItemCell::ITEM_LOCK:
    break;
  case ItemCell::ITEM_COMPLETE:
    {
      //Set_clist_column_style__complete(rowindex);
      //Set_clist_column__status(rowindex, _("DOWNLOAD COMPLETE"));
      if(delete_flag && (itemcell->Is_Partial() ||
			 itemcell->ret_Options_opt().ret_Delete_When_Finish() &&
			 (itemcell->ret_CRC_Type() != ItemCell::CRC_NONE && !itemcell->ret_Options_opt().ret_no_crc_checking() ||
			  !itemcell->ret_Options_opt().ret_Dont_Delete_Without_CRC()))) {
	if(GTK_CLIST(itemlistwidget)->selection != NULL) {
	  gtk_clist_unselect_row(GTK_CLIST(itemlistwidget), rowindex, 0);
	  if(GTK_CLIST(itemlistwidget)->selection == NULL) {
	    Set_sensitive__no_item_selected();
	  }
	}
	bool vadj_flag = false;
	float vadj = 0.0;
	GtkAdjustment *adj = NULL;
	if(gtk_clist_row_is_visible(GTK_CLIST(itemlistwidget), GTK_CLIST(itemlistwidget)->rows-1) == GTK_VISIBILITY_NONE) {
	  vadj_flag = true;
	  adj = gtk_clist_get_vadjustment(GTK_CLIST(itemlistwidget));
	  vadj = adj->value;
	}

	//gtk_clist_freeze(GTK_CLIST(itemlistwidget));
	
	Download_delete_sub(itemcell);
	//gtk_clist_thaw(GTK_CLIST(itemlistwidget));
	//cerr << "x" << endl;
	if(vadj_flag) {
	  gtk_adjustment_set_value(adj, vadj);
	}

	if(GTK_CLIST(itemlistwidget)->rows == 0) {
	  Set_sensitive__list_empty();
	}
      }
      break;
    }
  case ItemCell::ITEM_DOWNLOAD_AGAIN:
  case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
  case ItemCell::ITEM_DOWNLOAD:
    //Set_clist_column_style__download(rowindex);
    //Set_clist_column__status(rowindex, _("DOWNLOADING"));
    break;
  case ItemCell::ITEM_ERROR:
    //Set_clist_column_style__error(rowindex);
    //Set_clist_column__status(rowindex, _("DOWNLOAD ERROR"));
    break;
  case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    //Set_clist_column_style__divided(rowindex);
    //Set_clist_column__status(rowindex, _("DIVIDED"));
    break;
  case ItemCell::ITEM_STOP:
    //Set_clist_column_style__stop(rowindex);
    //Set_clist_column__status(rowindex, _("STOPPED"));
    break;
  case ItemCell::ITEM_CRCERROR:
    //Set_clist_column_style__crcerror(rowindex);
    //Set_clist_column__status(rowindex, _("CRC ERROR"));
    break;
  default:
    break;
  }
  //gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  //pthread_mutex_unlock(&itemlistlock);
}


void ItemStatusDynamic::set_UpdateFlag(unsigned int flag)
{
  update_flag = flag;
}

void ItemStatusDynamic::set_DeleteFlag(bool flag)
{
  delete_flag = flag;
}

