//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: download.cc,v 1.25 2001/03/20 08:47:47 tujikawa Exp $

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include "aria.h"
#include "ItemCell_HTTP.h"
#include "ItemCell_FTP.h"
#include "ThreadManager.h"
#include "ItemManager.h"
#include "AppOption.h"
#include "ItemList.h"
#include "CommandList.h"
#include "HistoryWindow.h"
extern char *const *environ;

extern int pipetogui[2];
extern pthread_mutex_t itemlistlock;
extern pthread_mutex_t guiupdate_lock;
extern pthread_cond_t itemlistnotempty;
extern GtkWidget *itemlistwidget;
extern ItemList *itemlist_global;
extern ItemCell *console_item;
extern ThreadManager *thread_manager;
extern ItemManager *item_manager;
extern CommandList command_list;
extern HistoryWindow *history_window;
extern AppOption *appoption;

extern bool CRC_check_main(ItemCell *itemcell);
extern bool Is_track_required();

void Send_report(MessageType reporttype, void *data)
{
  StatusReport statusreport;
  
  statusreport.reporttype = reporttype;
  statusreport.data = data;
  pthread_mutex_lock(&guiupdate_lock);
  write(pipetogui[1], &statusreport, sizeof(statusreport));
  pthread_mutex_unlock(&guiupdate_lock);
}

static void UNLOCK_AND_EXIT()
{
  pthread_mutex_unlock(&itemlistlock);
  pthread_exit(NULL);
}

static string create_commandline2(const string& command) {
  string home_dir = g_get_home_dir();
  string commandline = "cd "+home_dir+"; "+command;
  return commandline;
}
/*
static int Execute_commandline(ItemCell *itemcell, const string& commandline)
{
  int status;
  int pid = fork();
  if(pid == -1)
    return -1;
  if(pid == 0) {
    const char *argv[4];
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = commandline.c_str();
    argv[3] = 0;
    execve("/bin/sh", (char **)argv, environ);
    exit(127);
  } else if(pid < 0) {
    itemcell->Send_message_to_gui(_("cannot execute: ")+commandline, MSG_DOWNLOAD_ERROR);
  }
  while(1) {
    if(waitpid(pid, &status, 0) == -1) {
      if(errno != EINTR)
	return -1;
    } else return status;
  } 
}
*/

static void Download_success(ItemCell *itemcell)
{
  string line;

  if(itemcell->Is_Partial()) {
    line = _("a part of '")+itemcell->ret_Filename()+_("' download complete");
  } else {
    string filename;
    if(itemcell->ret_Filename().empty()) {
      filename = _("<directory>");
    } else {
      filename = itemcell->ret_Filename();
    }
    line = "'"+filename+_("' download complete");
    history_window->add_item(itemcell);
  }
  itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_SUCCESS);
  console_item->Send_message_to_gui(line, MSG_SYS_SUCCESS);
  //itemcell->set_Status(ItemCell::ITEM_COMPLETE);
  //itemcell->Close_Desc(); // moved to below

  itemcell->set_Status(ItemCell::ITEM_COMPLETE);
  itemcell->Close_Desc();

  itemcell->Send_status_complete();
  itemcell = NULL;
}

static void Download_partial_success(ItemCell *itemcell)
{


  string line = _("starting split download");
  itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
  itemcell->Close_Desc();
  //console_item->Send_message_to_gui(line, MSG_SYS_INFO);
  itemcell = NULL;
}

static void Download_stop(ItemCell *itemcell)
{
  string line = _("download stopped");
  itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
  itemcell->set_Status(ItemCell::ITEM_STOP);
  itemcell->Send_status();
  itemcell->Close_Desc();
}

static void Download_delete(ItemCell *itemcell)
{
  itemcell->Close_Desc();
  delete itemcell;
  itemcell = NULL;
}
  
static bool Download_error(ItemCell *itemcell)
{
  fd_set rfds;

  itemcell->Send_message_to_gui(_("wait for 10 seconds"), MSG_DOWNLOAD_ERROR);
  itemcell->Send_status();

  FD_ZERO(&rfds);
  FD_SET(itemcell->ret_Desc_r(), &rfds);

  struct timeval tv;
  tv.tv_sec = RETRYWAIT;
  tv.tv_usec = 0;

  int retval = select(itemcell->ret_Desc_r()+1, &rfds, NULL, NULL, &tv);
  if(retval && FD_ISSET(itemcell->ret_Desc_r(), &rfds)) {
    ItemCommand itemcommand;
    read(itemcell->ret_Desc_r(), &itemcommand, sizeof(struct ItemCommand));
    itemcell->set_Command(itemcommand);
    switch(itemcommand.commandtype) {
    case ItemCommand::COMMAND_STOP:
      itemcell->Close_Desc();
      itemcell->set_Status(ItemCell::ITEM_STOP);
      itemcell->Send_message_to_gui(_("download stopped"), MSG_DOWNLOAD_INFO);
      return true;
    case ItemCommand::COMMAND_DELETE:
      itemcell->Close_Desc();
      delete itemcell;
      itemcell = NULL;
      return true;
    case ItemCommand::COMMAND_HALT:
      itemcell->Close_Desc();
      pthread_exit(NULL);
    case ItemCommand::COMMAND_DOWNLOAD_AGAIN:
      itemcell->set_Status(ItemCell::ITEM_DOWNLOAD_AGAIN);
      itemcell->Send_message_to_gui(_("download again"), MSG_DOWNLOAD_INFO);
      return false;
    default:
      return true;
    }
  } else {
    return false;
  }
}

static void Maximum_retry_count_exceeded(ItemCell *itemcell)//, unsigned int& retry_repeat)
{
  /*
  ++retry_repeat;
  unsigned int retry_repeat_item;
  itemcell->get_Options_Lock();
  retry_repeat_item = itemcell->ret_Options_opt().ret_Retry_repeat();
  itemcell->release_Options_Lock();
  */
  //if(1 || retry_repeat  > retry_repeat_item) {
    string line = "'"+itemcell->ret_Filename()+_("' download aborted");
    itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_ERROR);
    itemcell->set_Status(ItemCell::ITEM_ERROR);
    itemcell->Send_status();
    itemcell->Close_Desc();
    /*
  } else {
    string line = "push back '"+itemcell->ret_Filename()+_("' to the end of the list");
    itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_ERROR);
    itemcell->set_Status(ItemCell::ITEM_ERROR);
    itemcell->Send_status();
    itemcell->Close_Desc();
  }
    */
  console_item->Send_message_to_gui(line, MSG_SYS_ERROR);
}

ItemCell *ftp2http(ItemCell *itemcell)
{
  ItemCell *itemcell_new = new ItemCell_HTTP(itemcell->ret_URL_opt(),
					     itemcell->ret_URL_Container_opt(),
					     itemcell->ret_Options_opt(),
					     "converted");
  return itemcell_new;
}

ItemCell *http2ftp(ItemCell *itemcell)
{
  //cerr << "in http2ftp" << endl;
  ItemCell *itemcell_new = new ItemCell_FTP(itemcell->ret_URL_opt(),
					    itemcell->ret_URL_Container_opt(),
					    itemcell->ret_Options_opt(),
					    "converted");
  return itemcell_new;
}

ItemCell *Protocol_conversion(ItemCell *itemcell)
{
  ItemCell *itemcell_new;
  if(itemcell->ret_URL_Container_opt().ret_Protocol() == "http:") {
    itemcell_new = ftp2http(itemcell);
  } else {
    itemcell_new = http2ftp(itemcell);
  }
  return itemcell_new;
}

void *Exec_and_wait_thread()
{
  int status;
  int pid = fork();

  string commandline = create_commandline2(appoption->ret_arb_command());

  try {
    if(pid == -1)
      throw 0;
    if(pid == 0) {
      const char *argv[4];
      argv[0] = "sh";
      argv[1] = "-c";
      argv[2] = commandline.c_str();
      argv[3] = 0;
      execve("/bin/sh", (char **)argv, environ);
      exit(0);
    } else if(pid < 0) {
      throw 0;
      //console_item->Send_message_to_gui(_("cannot execute: ")+commandline, MSG_SYS_ERROR);
    }
    while(1) {
      if(waitpid(pid, &status, 0) == -1) {
	if(errno != EINTR)
	  throw 0;
      } else throw 0;
    }
    throw 0;
  } catch (int err) {
    pthread_exit(NULL);
  }
}

void Do_something_on_event(ItemCommand::EventCause last_event)
{
  switch(last_event) {
  case ItemCommand::EV_INTERNAL:
  case ItemCommand::EV_TIMERINTER:
    // send message to gui to execute arbitrary command
    //cerr << "event must be: timer interruption or normal event" << endl;
    if(appoption->ret_use_arb_command() && appoption->ret_arb_command().size()) {
      string commandline = create_commandline2(appoption->ret_arb_command());
      console_item->Send_message_to_gui(_("executing program: ")+commandline, MSG_SYS_INFO);
      pthread_t do_anything_thread;
      pthread_create(&do_anything_thread, NULL, (void *(*)(void*))Exec_and_wait_thread, NULL);
    }
    break;
  default:
    //cerr << "event must be: user interaction or halt message from gui" << endl;
    break;
  }
}

ItemCell::DownloadStatusType Execute_command_list_entry(ItemCell *itemcell)
{
  ItemCell::DownloadStatusType retval = ItemCell::DLSUCCESS;

  Command command = command_list.search(itemcell->ret_Filename());
  if(command.Is_valid()) {
    string line = _("executing command entry '")+command.ret_command_name()+"'";
    itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
    string commandline = command.Create_commandline(itemcell);
    line = _("command line is '")+commandline+"'";
    itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
    
    //int status = Execute_commandline(itemcell, commandline);
    int status = command.Execute_commandline(itemcell);
    int ex_stat =  WIFEXITED(status);
    int return_stat;
    if(ex_stat) {
      return_stat = WEXITSTATUS(status);
      line = _("exit status is ")+itos(return_stat);
      itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
    } else {
      line = _("program exited abnormally");
      itemcell->Send_message_to_gui(line, MSG_DOWNLOAD_ERROR);
    }
    
    if(!command.Is_ignore_return_status()) {
      if(ex_stat && command.Is_in_succ_status_list(return_stat)) {
	itemcell->Send_message_to_gui(_("good exit status"), MSG_DOWNLOAD_SUCCESS);
      } else {
	itemcell->Send_message_to_gui(_("bad exit status"), MSG_DOWNLOAD_ERROR);
	itemcell->set_Command(ItemCell::DLERROR);
	retval = ItemCell::DLERROR;
	itemcell->set_Status(ItemCell::ITEM_EXECERROR);
      }
    } else {
      itemcell->Send_message_to_gui(_("exit status ignored"), MSG_DOWNLOAD_INFO);
    }
  }
  return retval;
}

void *Download_thread_main(bool autostart_flag)
{
  int retval;
  ItemCell *itemcell = NULL;

  ItemCommand::EventCause last_event = ItemCommand::EV_NOEVENT;

  nice(20);
  while(1) {
    pthread_mutex_lock(&itemlistlock);
    //thread_manager->Change_thread_state(pthread_self(), THREAD_WAIT);
    while(1) {
      if(thread_manager->Whether_halt()) {
	UNLOCK_AND_EXIT();
      } else if(thread_manager->Retire_thread_if_requested(pthread_self())) {
	//} else if(thread_manager->Whether_retire_is_required()) {
	//thread_manager->Retire_thread(pthread_self());
	UNLOCK_AND_EXIT();
      }
      if(autostart_flag) {
	itemcell = thread_manager->Get_next_item();
      } else {
	itemcell = NULL;
	autostart_flag = true;
      }
      if(itemcell != NULL) {
	thread_manager->Change_thread_state(pthread_self(), THREAD_ACTIVE);
	break;
      } else {
	thread_manager->Change_thread_state(pthread_self(), THREAD_WAIT);
	if(thread_manager->Is_all_threads_sleeping()) {	  
	  //cerr << "all thread sleeping ZZZzzzz..." << endl;
	  Do_something_on_event(last_event);
	}
	pthread_cond_wait(&itemlistnotempty, &itemlistlock);
	last_event = ItemCommand::EV_NOEVENT;
      }
    }
    pthread_mutex_unlock(&itemlistlock);
    itemcell->reset_Count();
    itemcell->Send_status();
    /*
    if(Is_track_required()) {
      int dst_row = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
      if(gtk_clist_row_is_visible(GTK_CLIST(itemlistwidget), dst_row) != GTK_VISIBILITY_FULL) {
	gtk_clist_moveto(GTK_CLIST(itemlistwidget), dst_row, 0, 0.0, 0.0);
      }
    }
    */
    //int retry_repeat = 0;
    while(1) {
      // update option values
      if(itemcell->ret_Status() == ItemCell::ITEM_INUSE_CONCAT) {
	itemcell->Send_message_to_gui(_("concatenating split files..."), MSG_DOWNLOAD_INFO);
	if(itemcell->Concatenate_partial_files()) {
	  retval = ItemCell::DLSUCCESS;
	  itemcell->Send_message_to_gui(_("concatenation successful"), MSG_DOWNLOAD_SUCCESS);
	  itemcell->set_Size_Current(itemcell->ret_Size_Total());
	} else {
	  retval = ItemCell::DLERROR;
	  itemcell->Send_message_to_gui(_("concatenation failed"), MSG_DOWNLOAD_ERROR);
	}
      } else {
	if(itemcell->Is_option_updated()) {
	  // acquire lock
	  itemcell->get_Options_Lock();
	  if(itemcell->ret_URL_Container().ret_Protocol() != itemcell->ret_URL_Container_opt().ret_Protocol()) {
	    ItemCell *itemcell_new = Protocol_conversion(itemcell);

	    item_manager->swap_item(itemcell_new, itemcell);

	    int rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
	    // release lock
	    itemcell->release_Options_Lock();
	    delete itemcell;

	    itemcell = itemcell_new;
	    itemcell->Open_Desc();
	    gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
	  } else {
	    itemcell->Apply_new_options();
	    // release lock
	    itemcell->release_Options_Lock();
	  }

	}
	retval = itemcell->Download_Main();
      }

      if(retval == ItemCell::DLSUCCESS || retval == ItemCell::DLSUCCESSALR) {
	if(itemcell->Post_process() != ItemCell::DLSUCCESS) {//crc check and etc
	  retval = ItemCell::DLERROR;
	  itemcell->set_Command(ItemCell::DLERROR);// fixed 2001/3/16
	  itemcell->set_Status(ItemCell::ITEM_CRCERROR);
	}
	// recursive download
	if(retval == ItemCell::DLSUCCESS && !itemcell->Is_Partial()) {
	  //pthread_mutex_lock(&itemlistlock);

	  if(itemcell->ret_Options().Is_in_activated_parse_target_list(itemcell->ret_Filename()) && !itemcell->ret_documentroot_dir().empty()) {

	    const URLcontainer& urlcon = itemcell->ret_URL_Container();
	    string base_url = urlcon.ret_Protocol()+"//"+urlcon.ret_Hostname()+urlcon.ret_Dir();
	    if(itemcell->ret_documentroot_dir().empty()) {
	      itemcell->set_documentroot_dir(itemcell->ret_Options().ret_Store_Dir());
	    }
	    if(itemcell->ret_root_url().empty()) {
	      itemcell->set_root_url(base_url+'/');
	    }
	    list<ItemCell*> item_list = itemlist_global->Recursive_add_http_item(itemcell);
	    if(item_list.size()) {
	      itemcell->Send_status_recursive(item_list);
	      sleep(2);
	    }
	  }
	  //pthread_mutex_unlock(&itemlistlock);
	}
      }
      if(retval == ItemCell::DLSUCCESS || retval == ItemCell::DLSUCCESSALR) {
	if(appoption->ret_use_commandlist() && !itemcell->Is_Partial()) {
	  itemcell->Send_status();
	  retval = Execute_command_list_entry(itemcell);
	}
      }

      if(retval == ItemCell::DLSUCCESS || retval == ItemCell::DLSUCCESSALR) {
	last_event = itemcell->ret_Dl_event();
	Download_success(itemcell);
	break;
      } else if(retval == ItemCell::DLPARTIALSUCCESS) {
	last_event = itemcell->ret_Dl_event();
	Download_partial_success(itemcell);
	break;
      } else if(retval == ItemCell::DLERROR) {
	if(itemcell->ret_Count() >= itemcell->ret_Options_opt().ret_Retry()) {
	  last_event = itemcell->ret_Dl_event();
	  Maximum_retry_count_exceeded(itemcell);//, retry_repeat);
	  break;
	} else {
	  if(Download_error(itemcell)) break;
	  last_event = itemcell->ret_Dl_event();
	  itemcell->inc_Count();// modified 2001/3/14
	}
      } else if(retval == ItemCell::DLERRORSTOP) {
	last_event = itemcell->ret_Dl_event();
	Maximum_retry_count_exceeded(itemcell);
	break;
      } else if(retval == ItemCell::DLSTOP) {
	last_event = itemcell->ret_Dl_event();
	Download_stop(itemcell);
	break;
      }	else if(retval == ItemCell::DLDELETE) {
	last_event = itemcell->ret_Dl_event();
	Download_delete(itemcell);
	break;
      } else if(retval == ItemCell::DLINTERNALAGAIN) {
	itemcell->set_Status(ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN);
      } else if(retval == ItemCell::DLAGAIN) {
	itemcell->set_Status(ItemCell::ITEM_DOWNLOAD_AGAIN);
      } else if(retval == ItemCell::DLHALT) {
	last_event = itemcell->ret_Dl_event();
	itemcell->Close_Desc();
	break;
      }
    }
  }
  return NULL;
}
