//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_download.cc,v 1.27 2001/03/22 08:42:18 tujikawa Exp $

#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <pair.h>
#include <list>
#include <algorithm>
#include "ThreadManager.h"
#include "ItemCell.h"
#include "ItemList.h"
#include "ItemManager.h"
#include "utils.h"
#include "Dialog.h"
#include "HistoryWindow.h"
#include "ShortCutKey.h"
#include "aria.h"
using namespace std;

extern bool CRC_check_main(ItemCell* itemcell);
extern void Set_sensitive__no_item_selected();
extern void Set_sensitive__list_empty();
extern void Set_clist_column__crc(int rowindex, const string& crc_string);
extern void Set_clist_column__status(int rowindex, const string& status);
extern void Set_clist_column_style__ready(int rowindex);
extern void Set_clist_column_style__complete(int rowindex);
extern void Set_clist_column_style__stop(int rowindex);
extern void Set_clist_column__icon(int rowindex, ItemCell::ItemStatusType status);
extern ItemManager *item_manager;
extern pthread_cond_t itemlistnotempty;
extern pthread_mutex_t itemlistlock;
extern GtkWidget* itemlistwidget;
extern GtkWidget* text;
extern GtkWidget* console_text;
extern ItemList* itemlist_global;
extern ItemCell* console_item;
extern ThreadManager* thread_manager;
extern AppOption *appoption;
extern Dialog *cdialog;
extern HistoryWindow *history_window;

static GtkWidget *start_item, *stop_item, *delete_item, *download_again_item;
static GtkWidget *clear_crc_item, *check_crc_item;
static GtkWidget *lock_item, *unlock_item;
static GtkWidget *start_all_item, *stop_all_item, *delete_all_item;
static GtkWidget *clear_crc_all_item;
static GtkWidget *delete_downloaded_item;
static GtkWidget *check_crc_downloaded_item;
static GtkWidget *delete_lock_item;
static GtkWidget *delete_error_item;
static GtkWidget *delete_stop_item;
static GtkWidget *delete_ready_item;
static GtkWidget *delete_error_stop_ready_item;
static GtkWidget *move_up_item, *move_down_item;
static GtkWidget *move_top_item, *move_bottom_item;
static GtkWidget *lock_error_item;
static int crcsockpair[2];
enum {
  DOWNLOAD_DEL_READY = 1 << 0,
  DOWNLOAD_DEL_STOP = 1 << 1,
  DOWNLOAD_DEL_ERROR = 1 << 2,
  DOWNLOAD_DEL_COMPLETE = 1 << 3,
  DOWNLOAD_DEL_LOCK = 1 << 4,
};
int state_flag;

void Download_set_sensitive__no_item_selected()
{
  gtk_widget_set_sensitive(start_item, FALSE);
  gtk_widget_set_sensitive(stop_item, FALSE);
  gtk_widget_set_sensitive(delete_item, FALSE);
  gtk_widget_set_sensitive(check_crc_item, FALSE);
  gtk_widget_set_sensitive(clear_crc_item, FALSE);
  gtk_widget_set_sensitive(download_again_item, FALSE);
  gtk_widget_set_sensitive(move_up_item, FALSE);
  gtk_widget_set_sensitive(move_down_item, FALSE);
  gtk_widget_set_sensitive(move_top_item, FALSE);
  gtk_widget_set_sensitive(move_bottom_item, FALSE);
  gtk_widget_set_sensitive(lock_item, FALSE);
  gtk_widget_set_sensitive(unlock_item, FALSE);
}

void Download_set_sensitive__items_selected()
{
  gtk_widget_set_sensitive(start_item, TRUE);
  gtk_widget_set_sensitive(stop_item, TRUE);
  gtk_widget_set_sensitive(delete_item, TRUE);
  gtk_widget_set_sensitive(check_crc_item, TRUE);
  gtk_widget_set_sensitive(clear_crc_item, TRUE);
  gtk_widget_set_sensitive(download_again_item, TRUE);
  gtk_widget_set_sensitive(move_up_item, TRUE);
  gtk_widget_set_sensitive(move_down_item, TRUE);
  gtk_widget_set_sensitive(move_top_item, TRUE);
  gtk_widget_set_sensitive(move_bottom_item, TRUE);
  gtk_widget_set_sensitive(lock_item, TRUE);
  gtk_widget_set_sensitive(unlock_item, TRUE);
}

void Download_set_sensitive__list_empty()
{
  gtk_widget_set_sensitive(start_all_item, FALSE);
  gtk_widget_set_sensitive(stop_all_item, FALSE);
  gtk_widget_set_sensitive(delete_all_item, FALSE);
  gtk_widget_set_sensitive(clear_crc_all_item, FALSE);
  gtk_widget_set_sensitive(delete_downloaded_item, FALSE);
  gtk_widget_set_sensitive(check_crc_downloaded_item, FALSE);
  gtk_widget_set_sensitive(delete_lock_item, FALSE);
  gtk_widget_set_sensitive(delete_error_item, FALSE);
  gtk_widget_set_sensitive(delete_stop_item, FALSE);
  gtk_widget_set_sensitive(delete_ready_item, FALSE);
  gtk_widget_set_sensitive(delete_error_stop_ready_item, FALSE);
  gtk_widget_set_sensitive(lock_error_item, FALSE);
}

void Download_set_sensitive__list_not_empty()
{
  gtk_widget_set_sensitive(start_all_item, TRUE);
  gtk_widget_set_sensitive(stop_all_item, TRUE);
  gtk_widget_set_sensitive(delete_all_item, TRUE);
  gtk_widget_set_sensitive(clear_crc_all_item, TRUE);
  gtk_widget_set_sensitive(delete_downloaded_item, TRUE);
  gtk_widget_set_sensitive(check_crc_downloaded_item, TRUE);
  gtk_widget_set_sensitive(delete_lock_item, TRUE);
  gtk_widget_set_sensitive(delete_error_item, TRUE);
  gtk_widget_set_sensitive(delete_stop_item, TRUE);
  gtk_widget_set_sensitive(delete_ready_item, TRUE);
  gtk_widget_set_sensitive(delete_error_stop_ready_item, TRUE);
  gtk_widget_set_sensitive(lock_error_item, TRUE);
}

// backup current selection list
static void Backup_selection_list(list<int>& selection_temp)
{
  for(GList* node = GTK_CLIST(itemlistwidget)->selection; node != NULL; node = g_list_next(node)) {
    selection_temp.push_back((int)node->data);
  }
}

// restore selection list
static void Restore_selection_list(const list<int>& selection_temp)
{
  list<int>::const_iterator sl_itr;
  for(sl_itr = selection_temp.begin(); sl_itr != selection_temp.end(); ++sl_itr) {
    gtk_clist_select_row(GTK_CLIST(itemlistwidget), *sl_itr, 0);
  }
}

enum DownloadListMoveType {
  DOWNLOAD_MOVE_TOP,
  DOWNLOAD_MOVE_BOTTOM,
  DOWNLOAD_MOVE_UP,
  DOWNLOAD_MOVE_DOWN
};

unsigned int find_min_value(GList *node)
{
  unsigned int min = GPOINTER_TO_UINT(node->data);
  node = g_list_next(node);
  while(node) {
    if(min > GPOINTER_TO_UINT(node->data)) {
      min = GPOINTER_TO_UINT(node->data);
    }
    node = g_list_next(node);
  }
  return min;
}

unsigned int find_max_value(GList *node)
{
  unsigned int max = GPOINTER_TO_UINT(node->data);
  node = g_list_next(node);
  while(node) {
    if(max < GPOINTER_TO_UINT(node->data)) {
      max = GPOINTER_TO_UINT(node->data);
    }
    node = g_list_next(node);
  }
  return max;
}

int gt_func(int x, int y)
{
  if(x < y) return 1;
  else return -1;
}

int lt_func(int x, int y)
{
  if(x > y) return 1;
  else return -1;
}

void Download_change_speed_sub(ItemCell *itemcell, float fspeed)
{
  switch(itemcell->ret_Status()) {
  case ItemCell::ITEM_DOWNLOAD:
    //case ItemCell::ITEM_DOWNLOAD_PARTIAL:
  case ItemCell::ITEM_INUSE:
  case ItemCell::ITEM_INUSE_CONCAT:
  case ItemCell::ITEM_INUSE_AGAIN:
  case ItemCell::ITEM_DOWNLOAD_AGAIN:
  case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
  case ItemCell::ITEM_CRCERROR:
  case ItemCell::ITEM_EXECERROR:
    {
      ItemCommand itemcommand;
      itemcommand.commandtype = ItemCommand::COMMAND_CHANGE_SPEED;
      itemcommand.eventtype = ItemCommand::EV_USERINTER;
      itemcommand.value = fspeed;
      write(itemcell->ret_Desc_w(), &itemcommand, sizeof(ItemCommand));
      break;
    }
  case ItemCell::ITEM_ERROR:
  case ItemCell::ITEM_STOP:
  case ItemCell::ITEM_LOCK:
  case ItemCell::ITEM_COMPLETE:
    itemcell->get_Options_Lock();
    itemcell->ret_Options_opt().set_speed_limit(fspeed);
    itemcell->ret_Options().set_speed_limit(fspeed);
    itemcell->release_Options_Lock();
    break;
  case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    itemcell->get_Options_Lock();
    itemcell->ret_Options_opt().set_speed_limit(fspeed);
    itemcell->ret_Options().set_speed_limit(fspeed);
    itemcell->release_Options_Lock();    
    for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
      Download_change_speed_sub((ItemCell*)*item_ptr, fspeed);
    }
    break;
  default:
    break;
  }
}

void Download_change_speed(float fspeed)
{

  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) {
    return;
  }
  pthread_mutex_lock(&itemlistlock);
  while(node) {
    int rowindex = GPOINTER_TO_INT(node->data);
    ItemCell *itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    Download_change_speed_sub(itemcell, fspeed);

    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  //cerr << "end" << endl;
}
//// modified 2000/3/1
void Download_move_sub(DownloadListMoveType mtype)
{
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL || g_list_length(node) == (unsigned int)GTK_CLIST(itemlistwidget)->rows) return;
  pthread_mutex_lock(&itemlistlock);
  //int destrow = rowindex;
  int diff;

  GList *localcopy = g_list_copy(node);
  switch(mtype) {
  case DOWNLOAD_MOVE_TOP: {
    localcopy = g_list_sort(localcopy, (int (*)(const void *, const void *))lt_func);
    diff = -GPOINTER_TO_UINT(localcopy->data);
    //destrow = 0;

    break;
  }
  case DOWNLOAD_MOVE_BOTTOM:
    localcopy = g_list_sort(localcopy, (int (*)(const void *, const void *))gt_func);
    diff = GTK_CLIST(itemlistwidget)->rows-1-GPOINTER_TO_UINT(localcopy->data);
    //destrow = GTK_CLIST(itemlistwidget)->rows-1;

    break;
  case DOWNLOAD_MOVE_UP: {
    localcopy = g_list_sort(localcopy, (int (*)(const void *, const void *))lt_func);
    if(GPOINTER_TO_UINT(localcopy->data) == 0) diff = 0;
    else diff = -1;

    //destrow = rowindex-1;
    break;
  }
  case DOWNLOAD_MOVE_DOWN:
    localcopy = g_list_sort(localcopy, (int (*)(const void *, const void *))gt_func);
    if(GPOINTER_TO_UINT(localcopy->data) == (unsigned int)GTK_CLIST(itemlistwidget)->rows-1) diff = 0;
    else diff = 1;
    //destrow = rowindex+1;
    break;
  default:
    break;
  }
  //localcopy = g_list_sort(node, (int (*)(const void *, const void *))lt_func);


  node = localcopy;
  while(diff && node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, rowindex+diff);


    node = g_list_next(node);
  }
  g_list_free(localcopy);
  pthread_mutex_unlock(&itemlistlock);
}


gboolean Download_moveup(GtkWidget *w, gpointer data)
{
  /*
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) return TRUE;
  pthread_mutex_lock(&itemlistlock);
  int rowindex = GPOINTER_TO_UINT(node->data);
  gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, rowindex-1);
  pthread_mutex_unlock(&itemlistlock);
  */
  Download_move_sub(DOWNLOAD_MOVE_UP);
  return TRUE;
}

gboolean Download_movedown(GtkWidget *w, gpointer data)
{
  /*
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) return TRUE;
  pthread_mutex_lock(&itemlistlock);
  int rowindex = GPOINTER_TO_UINT(node->data);
  gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, rowindex+1);
  pthread_mutex_unlock(&itemlistlock);
  */
  Download_move_sub(DOWNLOAD_MOVE_DOWN);
  return TRUE;
}

gboolean Download_movetop(GtkWidget *w, gpointer data)
{
  /*
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) return TRUE;
  pthread_mutex_lock(&itemlistlock);
  int rowindex = GPOINTER_TO_UINT(node->data);
  gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, 0);
  pthread_mutex_unlock(&itemlistlock);
  */
  Download_move_sub(DOWNLOAD_MOVE_TOP);
  return TRUE;
}

gboolean Download_movebottom(GtkWidget *w, gpointer data)
{
  /*
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) return TRUE;
  pthread_mutex_lock(&itemlistlock);
  int rowindex = GPOINTER_TO_UINT(node->data);
  gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, GTK_CLIST(itemlistwidget)->rows-1);
  pthread_mutex_unlock(&itemlistlock);
  */
  Download_move_sub(DOWNLOAD_MOVE_BOTTOM);
  return TRUE;
}

//
// 򤵤줿ƥΥɤ򳫻
//
static bool Download_start_sub(ItemCell *itemcell, int rowindex, bool all_flag = false)
{
  bool retval = false;
  switch(itemcell->ret_Status()) {
  case ItemCell::ITEM_ERROR:
    if(all_flag && appoption->ret_use_ignore_error_item()) {
      break;
    }
  case ItemCell::ITEM_STOP:
    {
      itemcell->set_Status(ItemCell::ITEM_READY);
      //Set_clist_column_style__ready(rowindex);
      Set_clist_column__icon(rowindex, ItemCell::ITEM_READY);
      Set_clist_column__status(rowindex, _("READY"));
      retval = true;
      break;
    }
  case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    {
      for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
	int rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), *item_ptr);
	if(Download_start_sub((ItemCell*)*item_ptr, rowindex)) {
	  retval = true;
	}
      }
      break;
    }
  case ItemCell::ITEM_READY://added 2001/3/21
  case ItemCell::ITEM_READY_CONCAT://added 2001/3/21
    retval = true;
    break;
  case ItemCell::ITEM_READY_AGAIN://added 2001/3/21
    //itemcell->set_Status(ItemCell::ITEM_READY);
    retval = true;
  default:
    break;
  }
  return retval;
}

gboolean Download_start(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) {
      pthread_mutex_unlock(&itemlistlock);
      return(TRUE);
  }
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  // int count added 2001/3/21
  int count = 0;
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);

    gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, 0);
    item_manager->move_to_front(itemcell);

    if(Download_start_sub(itemcell, 0)) {
      ++count;
    }
    node = g_list_next(node);
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  /*
  gtk_clist_moveto(GTK_CLIST(itemlistwidget),
		   0,
		   0,
		   0.0,
		   0.0);
  */
  // added 2001/3/21
  if(count) {
    pthread_cond_broadcast(&itemlistnotempty);
  }
  pthread_mutex_unlock(&itemlistlock);

  return(TRUE);
}

//
// CLIST itemlistwidget򤵤줿ƥκ
//
// ºݤκ¹Ԥ롣褦Ȥ륢ƥबitemcellǤ롣
// 褦ȤƤ륢ƥबåɤˤäƤ硢
// åɤ˥åꥹåɤΤۤǺƤ餦
bool Download_delete_sub(ItemCell* itemcell)
{
  if(item_manager->search_item(itemcell)) {
    int rowindex = gtk_clist_find_row_from_data((GtkCList *)itemlistwidget, itemcell);
    gtk_clist_remove(GTK_CLIST(itemlistwidget), rowindex); // remove from clist
    item_manager->unregist_item(itemcell);
    switch(itemcell->ret_Status()) {
    case ItemCell::ITEM_DOWNLOAD:
    case ItemCell::ITEM_DOWNLOAD_AGAIN:
    case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
    case ItemCell::ITEM_CRCERROR:
    case ItemCell::ITEM_EXECERROR://added 2001/3/16
    case ItemCell::ITEM_ERROR:
    case ItemCell::ITEM_INUSE:
    case ItemCell::ITEM_INUSE_AGAIN:
      //case ITEM_INUSE_CONCAT:
      {
	ItemCommand itemcommand;
	itemcommand.commandtype = ItemCommand::COMMAND_DELETE;
	itemcommand.eventtype = ItemCommand::EV_USERINTER;//hmmm, may be EV_APPINTER?
	if(write(itemcell->ret_Desc_w(), &itemcommand, sizeof(ItemCommand)) < 0) {
	  delete itemcell;// i'm not sure this line is whether safe or unsafe
	}
	break;
      }
    default:
      {
	delete itemcell;
	break;
      }
    }
    return true;
  } else {
    return false;
  }
}

gboolean Download_delete(GtkWidget* w, gpointer data)
{
  int count = 0;
  pthread_mutex_lock(&itemlistlock);
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  while (node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
      Download_delete_sub((ItemCell*)*item_ptr);
    }
    if(itemcell->Is_Partial()) {
      node = g_list_next(node);
    } else {
      if(Download_delete_sub(itemcell)) count++;
      node = GTK_CLIST(itemlistwidget)->selection;
    }
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  pthread_mutex_unlock(&itemlistlock);

  if(count > 0) {
    string line = itos(count)+_("item(s) deleted");
    console_item->Send_message_to_gui(line, MSG_SYS_INFO);
    Set_sensitive__no_item_selected();
    if(GTK_CLIST(itemlistwidget)->rows == 0) {
      Set_sensitive__list_empty();
    }
  }
  return TRUE;
}

gboolean Download_delete_ok(GtkWidget *w, GtkWidget *window)
{
  cdialog->hide();
  Download_delete(w, NULL);

  return TRUE;
}

gboolean Download_delete_c(GtkWidget *w, gpointer data)
{
  if(appoption->ret_conform_delete()) {
    cdialog->setup(_("Clear items"),
		   _("Are you sure to delete these items?"),
		   Download_delete_ok);
    cdialog->set_cancel_button_visible(false);
    cdialog->show();
  } else {
    Download_delete(w, data);
  }
  return TRUE;
}

//
// 򤵤줿ƥ٥
//
void Download_download_again_sub(ItemCell *itemcell)
{
  ItemCommand itemcommand;
  
  switch(itemcell->ret_Status()) {
  case ItemCell::ITEM_READY:
  case ItemCell::ITEM_READY_AGAIN:
  case ItemCell::ITEM_STOP:
  case ItemCell::ITEM_ERROR:
  case ItemCell::ITEM_COMPLETE:
    {
      // delete file
      // acquire lock
      itemcell->get_Options_Lock();
      string filename = itemcell->ret_Options_opt().ret_Store_Dir()+itemcell->ret_Filename();
      unlink(filename.c_str());

      if(itemcell->ret_Options_opt().ret_Divide() > 1) {
	for(int i = 0; i < (int)itemcell->ret_Options_opt().ret_Divide(); ++i) {
	  string filename = itemcell->ret_Options_opt().ret_Store_Dir()+itemcell->ret_Filename()+"."+itos(i);
	  unlink(filename.c_str());
	  string indexfilename = filename+".index";
	  unlink(indexfilename.c_str());
	}
      }
      // release lock
      itemcell->release_Options_Lock();

      itemcell->set_Status(ItemCell::ITEM_READY_AGAIN);
      itemcell->set_Size_Current(0);
      itemcell->set_Size_Total(0);
      itemcell->Send_status();


      break;
    }
  case ItemCell::ITEM_INUSE_AGAIN:
  case ItemCell::ITEM_CRCERROR: // added 2001/3/16
  case ItemCell::ITEM_EXECERROR:// added 2001/3/16
  case ItemCell::ITEM_DOWNLOAD:
  case ItemCell::ITEM_INUSE:
  case ItemCell::ITEM_DOWNLOAD_AGAIN:
  case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
    {
      itemcommand.commandtype = ItemCommand::COMMAND_DOWNLOAD_AGAIN;
      itemcommand.eventtype = ItemCommand::EV_USERINTER;
      write(itemcell->ret_Desc_w(), &itemcommand, sizeof(ItemCommand));
      break;
    }
  case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    {
      //礤...
      //ʬɤ򤷤Ƥ, ꤷʬǥ
      //ʤˤ, ٤Ƥʬɥƥब߾
      //(ITEM_READY, ITEM_STOP, ITEM_ERROR, ITEM_COMPLETE)λ˥
      //˥塼֥ɤľפ򤹤ˡޤ.
      //⤷, ĤǤƥब߾֤ˤʤ, ʬѹ,
      //֥ɤľפ򤷤Ƥ, ѹʬǥ
      //ľȤˤʤޤ.
      //ߤޤ..

      //check whether all splited items are halt state
      bool all_stop = false;
      for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
	ItemCell *itemcell_worker = (ItemCell*)*item_ptr;
	switch(itemcell_worker->ret_Status()) {
	case ItemCell::ITEM_READY:
	case ItemCell::ITEM_READY_AGAIN:
	case ItemCell::ITEM_STOP:
	case ItemCell::ITEM_ERROR:
	case ItemCell::ITEM_COMPLETE:
	  break;
	default:
	  {
	    all_stop = true;
	    break;
	  }
	}
	if(all_stop) break;
      }
      if(all_stop) {
	// if any is not in halt state,...
	for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
	  //int rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), (ItemCell*)*item_ptr);
	  Download_download_again_sub((ItemCell*)*item_ptr);
	}
	break;
      } else {
	// all items are in halt state..
	// delete file
	for(int i = 0; i < (int)itemcell->ret_Options().ret_Divide(); ++i) {
	  string filename = itemcell->ret_Options().ret_Store_Dir()+itemcell->ret_Filename()+"."+itos(i);
	  unlink(filename.c_str());
	  string indexfilename = filename+".index";
	  unlink(indexfilename.c_str());
	}
	// delete worker
	for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
	  Download_delete_sub(*item_ptr);
	}
	itemcell->Clear_worker();
	itemcell->set_Status(ItemCell::ITEM_READY_AGAIN);
	itemcell->set_Size_Current(0);
	itemcell->set_Size_Total(0);
	itemcell->Send_status();
	if(GTK_CLIST(itemlistwidget)->selection <= 0) {
	  Set_sensitive__no_item_selected();
	}
	if(GTK_CLIST(itemlistwidget)->rows == 0) {
	  Set_sensitive__list_empty();
	}
	break;
      }
      break;
    }
  default:
    break;
  }
}

gboolean Download_download_again(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList *node = GTK_CLIST(itemlistwidget)->selection;

  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);

    gtk_clist_row_move(GTK_CLIST(itemlistwidget), rowindex, 0);

    item_manager->move_to_front(itemcell);

    Download_download_again_sub(itemcell);
    node = g_list_next(node);   
  }
  /*
  gtk_clist_moveto(GTK_CLIST(itemlistwidget),
		   0,
		   0,
		   0.0,
		   0.0);
  */
  pthread_cond_broadcast(&itemlistnotempty);
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

//
// 򤵤줿ƥΥɤ
//
void Download_stop_sub(ItemCell *itemcell, int rowindex, bool all_flag = false, ItemCommand::EventCause eventtype = ItemCommand::EV_USERINTER)
{
  switch(itemcell->ret_Status()) {
    /* //modified 2001/3/2
  case ItemCell::ITEM_ERROR:
    if(all_flag && appoption->ret_use_ignore_error_item()) {
      break;
    }
    */
  case ItemCell::ITEM_DOWNLOAD:
  case ItemCell::ITEM_DOWNLOAD_AGAIN:
  case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
  case ItemCell::ITEM_CRCERROR:
  case ItemCell::ITEM_EXECERROR:
  case ItemCell::ITEM_INUSE:
  case ItemCell::ITEM_INUSE_AGAIN:
    {
      ItemCommand itemcommand;
      itemcommand.commandtype = ItemCommand::COMMAND_STOP;
      itemcommand.eventtype = eventtype;
      if(write(itemcell->ret_Desc_w(), &itemcommand, sizeof(ItemCommand)) < 0) {
	// may be item is not driven by download thread or,
	// item status is ITEM_CRCERROR or ITEM_ERROR and not are driven by
	// download thread
      } else {//modified 2001/3/2
	//Set_clist_column_style__stop(rowindex);
	Set_clist_column__icon(rowindex, ItemCell::ITEM_STOP);
	Set_clist_column__status(rowindex, _("STOPPED"));
      }
      break;
    }
  case ItemCell::ITEM_READY:
  case ItemCell::ITEM_READY_AGAIN:
    {
      itemcell->set_Status(ItemCell::ITEM_STOP);
      Set_clist_column__icon(rowindex, ItemCell::ITEM_STOP);
      //Set_clist_column_style__stop(rowindex);
      Set_clist_column__status(rowindex, _("STOPPED"));
      break;
    }
  case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    {
      for(list<ItemCell*>::const_iterator item_ptr = itemcell->ret_Worker_list().begin(); item_ptr != itemcell->ret_Worker_list().end(); ++item_ptr) {
	int rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), *item_ptr);
	Download_stop_sub(*item_ptr, rowindex);
      }
      break;
    }
  default:
    break;
  }
}

gboolean Download_stop(GtkWidget* w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    Download_stop_sub(itemcell, rowindex);
    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

static bool Do_crc_check(int rowindex, ItemCell *itemcell)
{
  bool retval = false;
  switch(itemcell->ret_Status()) {
  case ItemCell::ITEM_READY:
  case ItemCell::ITEM_READY_AGAIN:
  case ItemCell::ITEM_COMPLETE:
    //case ItemCell::ITEM_CRCERROR:
    try {
      if(itemcell->ret_CRC_Type() == ItemCell::CRC_NONE) {
	// do nothing
      } else if(CRC_check_main(itemcell)) {
	string line = _("CRC checking of '")+itemcell->ret_Filename()+_("' successful");
	console_item->Send_message_to_gui(line, MSG_SYS_SUCCESS);
	itemcell->set_Status(ItemCell::ITEM_COMPLETE);
	Set_clist_column__icon(rowindex, ItemCell::ITEM_COMPLETE);
	Set_clist_column__status(rowindex, _("DOWNLOAD COMPLETE"));
	history_window->add_item(itemcell);
	itemcell->Send_status_complete();
	retval = true;
      } else {
	string line = _("CRC checking of '")+itemcell->ret_Filename()+_("' failed");
	console_item->Send_message_to_gui(line, MSG_SYS_ERROR);
	//itemcell->set_Status(ItemCell::ITEM_CRCERROR);
	itemcell->set_Status(ItemCell::ITEM_ERROR);
	itemcell->Send_status();
      }
    }
    catch (ItemCell::ItemErrorType err) {
      itemcell->PERROR(err);
    }
    break;
  default:
    break;
  }
  return retval;
}

//
// 򤵤줿ƥCRCå
//

static void *Download_check_crc_sub()
{
  GList *node;
  ItemCell *itemcell;

  Socket socket(crcsockpair[1], Socket::DUPE);
  node = GTK_CLIST(itemlistwidget)->selection;
  while(node) {
    list<int> rfds_list;
    if(socket.is_readready(0, rfds_list) > 0) {
      char buffer[7];
      socket.Recv(buffer, sizeof(buffer));
      socket.Send("QUIT", 0);
      pthread_exit(NULL);
    }

    int rowindex = GPOINTER_TO_UINT(node->data);
    itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    node = g_list_next(node);
    Do_crc_check(rowindex, itemcell);
  }
  socket.Send("QUIT", 0);

  pthread_exit(NULL);
}

static void *Download_check_crc_downloaded_sub()
{
  Socket socket(crcsockpair[1], Socket::DUPE);
  int rowindex = 0;
  string temp_url;
  while(1) {
    //cerr << temp_url << endl;
    list<int> rfds_list;
    if(socket.is_readready(0, rfds_list) > 0) {
      char buffer[7];
      socket.Recv(buffer, sizeof(buffer));
      socket.Send("QUIT", 0);
      pthread_exit(NULL);
    }
    pthread_mutex_lock(&itemlistlock);
    if(rowindex == GTK_CLIST(itemlistwidget)->rows) {
      pthread_mutex_unlock(&itemlistlock);
      break;
    }
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    if(temp_url == itemcell->ret_URL()) {
      ++rowindex;
      itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    }
    temp_url = itemcell->ret_URL();
    pthread_mutex_unlock(&itemlistlock);
    if(!Do_crc_check(rowindex, itemcell)) {
      ++rowindex;
    }
  }
  socket.Send("QUIT", 0);

  pthread_exit(NULL);
}

static void Download_check_crc_end(void *w,
				   int sock_main,
				   GdkInputCondition dummy_cond)
{
  char buffer[5];
  Socket socket(sock_main, Socket::DUPE);
  socket.Recv(buffer, sizeof(buffer));
  cdialog->hide();
  if(GTK_CLIST(itemlistwidget)->selection == NULL) {
    Set_sensitive__no_item_selected();
  }
  if(GTK_CLIST(itemlistwidget)->rows == 0) {
    Set_sensitive__list_empty();
  }
}

static gboolean Download_check_crc_cancel(GtkWidget *w, GtkWidget *window)
{
  Socket socket(crcsockpair[0], Socket::DUPE);
  socket.Send("Cancel", 0);

  return TRUE;
}

gboolean Download_check_crc(GtkWidget* w, void *(*check_crc_func)())
{
  // cleaning up socket buffer
  Socket socket(crcsockpair[0], Socket::DUPE);
  list<int> fdlist;
  bool flag = true;
  while(flag) {
    fdlist.push_back(crcsockpair[1]);
    if(socket.is_readready(0, fdlist) > 0) {
      char buffer[7];
      if(Socket::is_set(crcsockpair[0], fdlist)) socket.Recv(buffer, sizeof(buffer));
      if(Socket::is_set(crcsockpair[1], fdlist)) recv(crcsockpair[1], buffer, sizeof(buffer), 0);
      fdlist.clear();
    } else {
      flag = false;
    }
  }
  cdialog->setup(_("CRC check"),
		 _("Cheking CRC..."),
		 NULL,
		 NULL,
		 Download_check_crc_cancel);
  cdialog->set_yes_button_visible(false);
  cdialog->set_no_button_visible(false);
  cdialog->show();

  pthread_t crc_thread;
  pthread_create(&crc_thread, (pthread_attr_t *)NULL,
		 (void *(*)(void *))check_crc_func,
		 (void *)NULL);

  return TRUE;
}

//
// 򤵤줿ƥCRC
//
gboolean Download_clear_crc(GtkWidget* w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->set_CRC_Type(ItemCell::CRC_NONE);
    Set_clist_column__crc(rowindex, "");

    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

//
// ٤ƤΥƥCRC
//
static gboolean Download_clear_crc_all(GtkWidget* w, gpointer data)
{
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  list<int> selection_temp;
  Backup_selection_list(selection_temp);

  gtk_clist_select_all(GTK_CLIST(itemlistwidget));
  Download_clear_crc(w, data);
  gtk_clist_unselect_all(GTK_CLIST(itemlistwidget));

  Restore_selection_list(selection_temp);
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  return TRUE;
}

//
// ٤߾֤ΥƥԵ(READY)ˤ, ɤ򳫻
//
gboolean Download_start_all(GtkWidget* w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  list<int> selection_temp;
  Backup_selection_list(selection_temp);

  gtk_clist_select_all(GTK_CLIST(itemlistwidget));
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) {
      pthread_mutex_unlock(&itemlistlock);
      return(TRUE);
  }
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    Download_start_sub(itemcell, rowindex, true);
    node = g_list_next(node);
  }
  gtk_clist_unselect_all(GTK_CLIST(itemlistwidget));

  Restore_selection_list(selection_temp);

  gtk_clist_thaw(GTK_CLIST(itemlistwidget));

  pthread_cond_broadcast(&itemlistnotempty);
  pthread_mutex_unlock(&itemlistlock);
  return(TRUE);
}

//
// ٤ƤΥƥΥɤľ
//
gboolean Download_download_again_all(GtkWidget* w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  list<int> selection_temp;
  Backup_selection_list(selection_temp);

  gtk_clist_select_all(GTK_CLIST(itemlistwidget));
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) {
      pthread_mutex_unlock(&itemlistlock);
      return(TRUE);
  }
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    Download_download_again_sub(itemcell);
    node = g_list_next(node);
  }
  gtk_clist_unselect_all(GTK_CLIST(itemlistwidget));

  Restore_selection_list(selection_temp);

  gtk_clist_thaw(GTK_CLIST(itemlistwidget));

  pthread_cond_broadcast(&itemlistnotempty);
  pthread_mutex_unlock(&itemlistlock);
  return(TRUE);
}

//
// ٤ƤΥƥ߾֤ˤ
//
gboolean Download_stop_all(GtkWidget* w, gpointer data)
{
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  list<int> selection_temp;
  Backup_selection_list(selection_temp);

  gtk_clist_select_all(GTK_CLIST(itemlistwidget));

  pthread_mutex_lock(&itemlistlock);
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    if(w == NULL) {
      Download_stop_sub(itemcell, rowindex, true, ItemCommand::EV_TIMERINTER);
    } else {
      Download_stop_sub(itemcell, rowindex, true);
    }
    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  gtk_clist_unselect_all(GTK_CLIST(itemlistwidget));

  Restore_selection_list(selection_temp);

  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  return(TRUE);
}

//
// ٤ƤΥƥ
//
gboolean Download_delete_all(GtkWidget* w, gpointer data)
{
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  gtk_clist_select_all(GTK_CLIST(itemlistwidget));
  Download_delete(w, data);
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  return(TRUE);
}

gboolean Download_delete_all_ok(GtkWidget *w, GtkWidget *window)
{
  cdialog->hide();
  Download_delete_all(w, NULL);

  return TRUE;
}

gboolean Download_delete_all_c(GtkWidget *w, gpointer data)
{
  if(appoption->ret_conform_delete()) {
    cdialog->setup(_("Clear items"),
		   _("Are you sure to delete these items?"),
		   Download_delete_all_ok);
    cdialog->set_cancel_button_visible(false);
    cdialog->show();
  } else {
    Download_delete_all(w, data);
  }
  return TRUE;
}
//
// ٤ƤΥɺѥƥ
//
/*
gboolean Download_delete_downloaded_item(GtkWidget* w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  int rowindex = 0;
  while(1) {
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    if(itemcell == NULL) break;
    if(itemcell->ret_Status() == ITEM_COMPLETE) {
      Download_delete_sub(itemcell);
    } else {
      ++rowindex;
    }
  }
  if(GTK_CLIST(itemlistwidget)->selection == NULL) {
    Set_sensitive__no_item_selected();
  }
  if(GTK_CLIST(itemlistwidget)->rows == 0) {
    Set_sensitive__list_empty();
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}
*/

void Download_delete_state_item(int state_flag)
{
  pthread_mutex_lock(&itemlistlock);
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  int rowindex = 0;
  int count = 0;
  while(1) {
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    if(itemcell == NULL) break;
    bool del_flag = false;
    switch(itemcell->ret_Status()) {
    case ItemCell::ITEM_ERROR:
      //case ItemCell::ITEM_CRCERROR:
      if(state_flag & DOWNLOAD_DEL_ERROR) {
	del_flag = true;
	Download_delete_sub(itemcell);
      }
      break;
    case ItemCell::ITEM_READY:
    case ItemCell::ITEM_READY_AGAIN:
    case ItemCell::ITEM_READY_CONCAT:
      if(state_flag & DOWNLOAD_DEL_READY) {
	del_flag = true;
	Download_delete_sub(itemcell);
      }
      break;
    case ItemCell::ITEM_STOP:
      if(state_flag & DOWNLOAD_DEL_STOP) {
	del_flag = true;
	Download_delete_sub(itemcell);
      }
      break;
    case ItemCell::ITEM_LOCK:
      if(state_flag & DOWNLOAD_DEL_LOCK) {
	del_flag = true;
	Download_delete_sub(itemcell);
      }
      break;
    case ItemCell::ITEM_COMPLETE:
      if(state_flag & DOWNLOAD_DEL_COMPLETE) {
	del_flag = true;
	Download_delete_sub(itemcell);
      }
    default:
      break;
    }
    if(!del_flag) {
      ++rowindex;
    } else {
      ++count;
    }
  }
  if(count > 0) {
    string line = itos(count)+_("item(s) deleted");
    console_item->Send_message_to_gui(line, MSG_SYS_INFO);
    if(GTK_CLIST(itemlistwidget)->selection == NULL) {
      Set_sensitive__no_item_selected();
    }
    if(GTK_CLIST(itemlistwidget)->rows == 0) {
      Set_sensitive__list_empty();
    }
  }
  if(GTK_CLIST(itemlistwidget)->selection == NULL) {
    Set_sensitive__no_item_selected();
  }
  if(GTK_CLIST(itemlistwidget)->rows == 0) {
    Set_sensitive__list_empty();
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  pthread_mutex_unlock(&itemlistlock);
}

gboolean Download_delete_state_item_ok(GtkWidget *w, GtkWidget *window)
{
  cdialog->hide();
  Download_delete_state_item(state_flag);

  return TRUE;
}

void Download_delete_state_item_c()
{
  if(appoption->ret_conform_delete()) {
    cdialog->setup(_("Clear items"),
		   _("Are you sure to delete these items?"),
		   Download_delete_state_item_ok);
    cdialog->set_cancel_button_visible(false);
    cdialog->show();
  } else {
    Download_delete_state_item(state_flag);
  }
}

gboolean Download_delete_stop_item(GtkWidget *w, gpointer data)
{
  state_flag = DOWNLOAD_DEL_STOP;
  Download_delete_state_item_c();

  return TRUE;
}

gboolean Download_delete_error_item(GtkWidget *w, gpointer data)
{
  state_flag = DOWNLOAD_DEL_ERROR;
  Download_delete_state_item_c();

  return TRUE;
}

gboolean Download_delete_ready_item(GtkWidget *w, gpointer data)
{
  state_flag = DOWNLOAD_DEL_READY;
  Download_delete_state_item_c();

  return TRUE;
}

gboolean Download_delete_downloaded_item(GtkWidget *w, gpointer data)
{
  state_flag = DOWNLOAD_DEL_COMPLETE;
  Download_delete_state_item_c();

  return TRUE;
}

gboolean Download_delete_locked_item(GtkWidget *w, gpointer data)
{
  state_flag = DOWNLOAD_DEL_LOCK;
  Download_delete_state_item_c();

  return TRUE;
}

gboolean Download_delete_error_stop_ready_item(GtkWidget *w, gpointer data)
{
  state_flag = DOWNLOAD_DEL_ERROR | DOWNLOAD_DEL_READY | DOWNLOAD_DEL_STOP;
  Download_delete_state_item_c();

  return TRUE;
}

gboolean Download_unlock_item(GtkWidget *w, gpointer data)
{
  try{
    pthread_mutex_lock(&itemlistlock);
    GList *node = GTK_CLIST(itemlistwidget)->selection;
    if(!node) throw 0;
    while(node) {
      int rowindex GPOINTER_TO_UINT(node->data);
      ItemCell *itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
      switch(itemcell->ret_Status()) {
      case ItemCell::ITEM_LOCK:
	itemcell->set_Status(ItemCell::ITEM_READY);
	Set_clist_column__icon(rowindex, ItemCell::ITEM_READY);
	break;
      default:
	break;
      }
      node = g_list_next(node);
    }
    pthread_mutex_unlock(&itemlistlock);
  } catch (int err) {
    pthread_mutex_unlock(&itemlistlock);
    return TRUE;
  }
  return TRUE;
}

gboolean Download_lock_error_item(GtkWidget *w, gpointer data)
{
  try{
    pthread_mutex_lock(&itemlistlock);
    for(int rowindex = 0; rowindex < GTK_CLIST(itemlistwidget)->rows; ++rowindex) {
      ItemCell *itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
      switch(itemcell->ret_Status()) {
      case ItemCell::ITEM_ERROR:
	itemcell->set_Status(ItemCell::ITEM_LOCK);
	Set_clist_column__icon(rowindex, ItemCell::ITEM_LOCK);
	break;
      default:
	break;
      }
    }
    pthread_mutex_unlock(&itemlistlock);
  } catch (int err) {
    pthread_mutex_unlock(&itemlistlock);
    return TRUE;
  }
  return TRUE;
}

gboolean Download_lock_item(GtkWidget *w, gpointer data)
{
  try{
    pthread_mutex_lock(&itemlistlock);
    GList *node = GTK_CLIST(itemlistwidget)->selection;
    if(!node) throw 0;
    while(node) {
      int rowindex = GPOINTER_TO_UINT(node->data);
      ItemCell *itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
      switch(itemcell->ret_Status()) {
      case ItemCell::ITEM_READY:
      case ItemCell::ITEM_READY_AGAIN:
      case ItemCell::ITEM_READY_CONCAT:
      case ItemCell::ITEM_STOP:
      case ItemCell::ITEM_ERROR:
	itemcell->set_Status(ItemCell::ITEM_LOCK);
	Set_clist_column__icon(rowindex, ItemCell::ITEM_LOCK);
	break;
      default:
	break;
      }
      node = g_list_next(node);
    }
    pthread_mutex_unlock(&itemlistlock);
  } catch (int err) {
    pthread_mutex_unlock(&itemlistlock);
    return TRUE;
  }
  return TRUE;
}

//
// ɥ˥塼
//
void Create_download_menu(GtkWidget* menu_bar, GtkAccelGroup* accel_group)
{
  GtkWidget *download_menu;
  GtkWidget *download_item;
  GtkWidget *sep_download;
  GtkWidget *sep_download_2;
  GtkWidget *sep_download_3;
  GtkWidget *sep_download_4;
  
  download_menu = gtk_menu_new();
  // menu for selected items
  start_item = gtk_menu_item_new_with_label(_("Start"));
  stop_item = gtk_menu_item_new_with_label(_("Stop"));
  delete_item = gtk_menu_item_new_with_label(_("Clear"));
  clear_crc_item = gtk_menu_item_new_with_label(_("Clear CRC"));
  check_crc_item = gtk_menu_item_new_with_label(_("Check CRC"));
  download_again_item = gtk_menu_item_new_with_label(_("Download again"));
  lock_item = gtk_menu_item_new_with_label(_("Lock"));
  unlock_item = gtk_menu_item_new_with_label(_("Unlock"));

  sep_download = gtk_menu_item_new();
  // menu for all items
  start_all_item = gtk_menu_item_new_with_label(_("Start All"));
  stop_all_item = gtk_menu_item_new_with_label(_("Stop All"));
  delete_all_item = gtk_menu_item_new_with_label(_("Clear All"));
  clear_crc_all_item = gtk_menu_item_new_with_label(_("Clear CRC All"));

  sep_download_2 = gtk_menu_item_new();
  // menu for moving item
  move_up_item = gtk_menu_item_new_with_label(_("Move item up"));
  move_down_item = gtk_menu_item_new_with_label(_("Move item down"));
  move_top_item = gtk_menu_item_new_with_label(_("Move item to top"));
  move_bottom_item = gtk_menu_item_new_with_label(_("Move item to bottom"));

  sep_download_3 = gtk_menu_item_new();
  // menu for completed item
  delete_downloaded_item = gtk_menu_item_new_with_label(_("Clear downloaded item"));
  check_crc_downloaded_item = gtk_menu_item_new_with_label(_("Check downloaded items' CRC"));

  sep_download_4 = gtk_menu_item_new();
  // misc menu
  lock_error_item = gtk_menu_item_new_with_label(_("Lock error items"));
  delete_lock_item = gtk_menu_item_new_with_label(_("Clear locked items"));
  delete_error_item = gtk_menu_item_new_with_label(_("Clear error items"));
  delete_stop_item = gtk_menu_item_new_with_label(_("Clear stopped items"));
  delete_ready_item = gtk_menu_item_new_with_label(_("Clear ready items"));
  delete_error_stop_ready_item = gtk_menu_item_new_with_label(_("Clear error, stopped, ready items"));

  gtk_menu_append(GTK_MENU(download_menu), start_item);
  gtk_menu_append(GTK_MENU(download_menu), stop_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_item);
  gtk_menu_append(GTK_MENU(download_menu), clear_crc_item);
  gtk_menu_append(GTK_MENU(download_menu), check_crc_item);
  gtk_menu_append(GTK_MENU(download_menu), download_again_item);
  gtk_menu_append(GTK_MENU(download_menu), lock_item);
  gtk_menu_append(GTK_MENU(download_menu), unlock_item);
  gtk_menu_append(GTK_MENU(download_menu), sep_download);
  gtk_menu_append(GTK_MENU(download_menu), start_all_item);
  gtk_menu_append(GTK_MENU(download_menu), stop_all_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_all_item);
  gtk_menu_append(GTK_MENU(download_menu), clear_crc_all_item);
  gtk_menu_append(GTK_MENU(download_menu), sep_download_2);
  gtk_menu_append(GTK_MENU(download_menu), move_up_item);
  gtk_menu_append(GTK_MENU(download_menu), move_down_item);
  gtk_menu_append(GTK_MENU(download_menu), move_top_item);
  gtk_menu_append(GTK_MENU(download_menu), move_bottom_item);
  gtk_menu_append(GTK_MENU(download_menu), sep_download_3);
  gtk_menu_append(GTK_MENU(download_menu), delete_downloaded_item);
  gtk_menu_append(GTK_MENU(download_menu), check_crc_downloaded_item);
  gtk_menu_append(GTK_MENU(download_menu), sep_download_4);
  gtk_menu_append(GTK_MENU(download_menu), lock_error_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_lock_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_error_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_stop_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_ready_item);
  gtk_menu_append(GTK_MENU(download_menu), delete_error_stop_ready_item);
  // signal handling
  gtk_signal_connect(GTK_OBJECT(start_item), "activate",
		     GTK_SIGNAL_FUNC(Download_start), NULL);
  gtk_signal_connect(GTK_OBJECT(stop_item), "activate",
		     GTK_SIGNAL_FUNC(Download_stop), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_c), NULL);
  gtk_signal_connect(GTK_OBJECT(clear_crc_item), "activate",
		     GTK_SIGNAL_FUNC(Download_clear_crc), NULL);
  gtk_signal_connect(GTK_OBJECT(check_crc_item), "activate",
		     GTK_SIGNAL_FUNC(Download_check_crc), (void *)Download_check_crc_sub);
  gtk_signal_connect(GTK_OBJECT(download_again_item), "activate",
		     GTK_SIGNAL_FUNC(Download_download_again), NULL);
  gtk_signal_connect(GTK_OBJECT(lock_item), "activate",
		     GTK_SIGNAL_FUNC(Download_lock_item), NULL);
  gtk_signal_connect(GTK_OBJECT(unlock_item), "activate",
		     GTK_SIGNAL_FUNC(Download_unlock_item), NULL);
  gtk_signal_connect(GTK_OBJECT(start_all_item), "activate",
		     GTK_SIGNAL_FUNC(Download_start_all), NULL);
  gtk_signal_connect(GTK_OBJECT(stop_all_item), "activate",
		     GTK_SIGNAL_FUNC(Download_stop_all), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_all_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_all_c), NULL);
  gtk_signal_connect(GTK_OBJECT(clear_crc_all_item), "activate",
		     GTK_SIGNAL_FUNC(Download_clear_crc_all), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_downloaded_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_downloaded_item), NULL);

  gtk_signal_connect(GTK_OBJECT(lock_error_item), "activate",
		     GTK_SIGNAL_FUNC(Download_lock_error_item), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_lock_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_locked_item), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_error_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_error_item), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_stop_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_stop_item), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_ready_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_ready_item), NULL);
  gtk_signal_connect(GTK_OBJECT(delete_error_stop_ready_item), "activate",
		     GTK_SIGNAL_FUNC(Download_delete_error_stop_ready_item), NULL);

  gtk_signal_connect(GTK_OBJECT(check_crc_downloaded_item), "activate",
		     GTK_SIGNAL_FUNC(Download_check_crc), (void *)Download_check_crc_downloaded_sub);

  gtk_signal_connect(GTK_OBJECT(move_up_item), "activate",
		     GTK_SIGNAL_FUNC(Download_moveup), NULL);
  gtk_signal_connect(GTK_OBJECT(move_down_item), "activate",
		     GTK_SIGNAL_FUNC(Download_movedown), NULL);
  gtk_signal_connect(GTK_OBJECT(move_top_item), "activate",
		     GTK_SIGNAL_FUNC(Download_movetop), NULL);
  gtk_signal_connect(GTK_OBJECT(move_bottom_item), "activate",
		     GTK_SIGNAL_FUNC(Download_movebottom), NULL);

  // accelerator key
  gtk_widget_add_accelerator(start_item,
			     "activate",
			     accel_group,
			     SC_START,
			     SCM_START,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(stop_item,
			     "activate",
			     accel_group,
			     SC_STOP,
			     SCM_STOP,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(delete_item,
			     "activate",
			     accel_group,
			     SC_DELETE,
			     SCM_DELETE,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(lock_item,
			     "activate",
			     accel_group,
			     SC_LOCK,
			     SCM_LOCK,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(unlock_item,
			     "activate",
			     accel_group,
			     SC_UNLOCK,
			     SCM_UNLOCK,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(start_all_item,
			     "activate",
			     accel_group,
			     SC_START_ALL,
			     SCM_START_ALL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(stop_all_item,
			     "activate",
			     accel_group,
			     SC_STOP_ALL,
			     SCM_STOP_ALL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(delete_all_item,
			     "activate",
			     accel_group,
			     SC_DELETE_ALL,
			     SCM_DELETE_ALL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(download_again_item,
			     "activate",
			     accel_group,
			     SC_DOWNLOADAGAIN,
			     SCM_DOWNLOADAGAIN,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(move_up_item,
			     "activate",
			     accel_group,
			     SC_MOVEUP,
			     SCM_MOVEUP,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(move_down_item,
			     "activate",
			     accel_group,
			     SC_MOVEDOWN,
			     SCM_MOVEDOWN,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(move_top_item,
			     "activate",
			     accel_group,
			     SC_MOVETOP,
			     SCM_MOVETOP,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(move_bottom_item,
			     "activate",
			     accel_group,
			     SC_MOVEBOTTOM,
			     SCM_MOVEBOTTOM,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(delete_downloaded_item,
			     "activate",
			     accel_group,
			     SC_DELETEDOWNLOADED,
			     SCM_DELETEDOWNLOADED,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(lock_error_item,
			     "activate",
			     accel_group,
			     SC_LOCKERROR,
			     SCM_LOCKERROR,
			     GTK_ACCEL_VISIBLE);


  gtk_widget_show(start_item);
  gtk_widget_show(stop_item);
  gtk_widget_show(delete_item);
  gtk_widget_show(clear_crc_item);
  gtk_widget_show(check_crc_item);
  gtk_widget_show(download_again_item);
  gtk_widget_show(lock_item);
  gtk_widget_show(unlock_item);
  gtk_widget_show(sep_download);
  gtk_widget_show(start_all_item);
  gtk_widget_show(stop_all_item);
  gtk_widget_show(delete_all_item);
  gtk_widget_show(clear_crc_all_item);
  gtk_widget_show(sep_download_2);
  gtk_widget_show(move_up_item);
  gtk_widget_show(move_down_item);
  gtk_widget_show(move_top_item);
  gtk_widget_show(move_bottom_item);
  gtk_widget_show(sep_download_3);
  gtk_widget_show(delete_downloaded_item);
  gtk_widget_show(lock_error_item);
  gtk_widget_show(delete_lock_item);
  gtk_widget_show(delete_error_item);
  gtk_widget_show(delete_stop_item);
  gtk_widget_show(delete_ready_item);
  gtk_widget_show(delete_error_stop_ready_item);
  gtk_widget_show(sep_download_4);
  gtk_widget_show(check_crc_downloaded_item);

  // set sensitivity
  Download_set_sensitive__no_item_selected();
  Download_set_sensitive__list_empty();

  download_item = gtk_menu_item_new_with_label(_("Download"));
  gtk_widget_show(download_item);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(download_item), download_menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), download_item);

  if(socketpair(AF_UNIX, SOCK_STREAM, 0, crcsockpair) < 0) {

  }
  gdk_input_add(crcsockpair[0], GDK_INPUT_READ, Download_check_crc_end, NULL);  
}
