//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_edit.cc,v 1.29 2001/03/15 05:35:38 tujikawa Exp $

#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include "aria.h"
#include "ItemCell_HTTP.h"
#include "ItemCell_FTP.h"
#include "ThreadManager.h"
#include "ItemManager.h"
#include "PasteWindow.h"
#include "CRCList.h"
#include "AppOption.h"
#include "utils.h"
#include "aria_mime.h"
#include "ShortCutKey.h"
using namespace std;

extern void Set_sensitive__list_not_empty();
extern void Set_sensitive__items_selected();
extern void Set_clist_column__crc(int rowindex, const string& crc_string);
extern void Set_clist_column__progress(int rowindex, int progress);
extern void Set_clist_column__icon(int rowindex , ItemCell::ItemStatusType status);

extern unsigned int Find_nearest_selected_row(GtkWidget *clist, unsigned int row_current);

extern pthread_cond_t itemlistnotempty;
extern pthread_mutex_t itemlistlock;

extern GtkWidget *itemlistwidget, *text, *console_text;
extern ItemCell *console_item;

extern AppOption *appoption;
extern ThreadManager *thread_manager;
extern ItemManager *item_manager;
extern ItemManager *item_manager_paste;
PasteWindow *paste_window;

// select all item in download list
static gboolean Edit_select_all(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  gtk_clist_select_all(GTK_CLIST(itemlistwidget));
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  if(node != NULL) {
    Set_sensitive__items_selected();
  }
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

// clear system log
static gboolean Edit_clear_syslog(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  console_item->Clear_log();
  gtk_text_freeze(GTK_TEXT(console_text));
  unsigned int length = gtk_text_get_length(GTK_TEXT(console_text));
  gtk_text_set_point(GTK_TEXT(console_text), length);
  gtk_text_backward_delete(GTK_TEXT(console_text), length);
  gtk_text_thaw(GTK_TEXT(console_text));
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

void Add_new_item_to_downloadlist(ItemCell *itemcell)
{
  char *clist_item[TOTALCOL];
  /*
  //modified start
  for(int i = 0; i < TOTALCOL; ++i) {
    clist_item[i] = NULL;
  }
  string crc_string;
  switch(itemcell->ret_CRC_Type()) {
  case CRC_16:
    crc_string = itos_hex(itemcell->ret_CRC(), 4, '0');
    break;
  case CRC_32:
    crc_string = itos_hex(itemcell->ret_CRC(), 8, '0');
    break;
  default:
    break;
  }
  clist_item[COL_CRC] = g_strdup(crc_string.c_str());

  int rowindex = gtk_clist_append(GTK_CLIST(itemlistwidget), clist_item);
  gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
  itemcell->set_id(-1);

  item_manager->regist_item_back(itemcell);

  //modified end
  ItemStatusDynamic itemstatus(itemcell,
			       itemcell->ret_Status(),
			       0,
			       0);
  itemstatus.Update();
  delete [] clist_item[COL_CRC];
  */
  clist_item[COL_ICON] = NULL;
  if(itemcell->ret_Filename().empty()) {
    clist_item[COL_FILENAME] = g_strdup(_("<directory>"));
  } else {
    clist_item[COL_FILENAME] = g_strdup(itemcell->ret_Filename().c_str());
  }
  clist_item[COL_CURSIZE] = "0";
  clist_item[COL_TOTSIZE] = _("unknown");
  clist_item[COL_PROGRESS] = NULL;//"";
  clist_item[COL_RETRY] = g_strdup_printf("0/%d", itemcell->ret_Options().ret_Retry());
  int n_rec;
  if(itemcell->ret_URL_Container().ret_Protocol() == "http:") {
    n_rec = itemcell->ret_Options().ret_recurse_count();
  } else {
    n_rec = itemcell->ret_Options().ret_FTP_recurse_count();
  }
  clist_item[COL_REC] = g_strdup_printf("%d", n_rec);
  clist_item[COL_SPEED] = "";
  clist_item[COL_RTIME] = "";
  string crc_string;
  switch(itemcell->ret_CRC_Type()) {
  case ItemCell::CRC_16:
    crc_string = itos_hex(itemcell->ret_CRC(), 4, '0');
    break;
  case ItemCell::CRC_32:
    crc_string = itos_hex(itemcell->ret_CRC(), 8, '0');
    break;
  default:
    break;
  }
  clist_item[COL_CRC] = g_strdup(crc_string.c_str());
  clist_item[COL_STATUS] = _("READY");
  clist_item[COL_SAVE] = g_strdup(itemcell->ret_Options().ret_Store_Dir().c_str());
  clist_item[COL_URL] = g_strdup(itemcell->ret_URL().c_str());
  unsigned int rowindex = gtk_clist_append(GTK_CLIST(itemlistwidget), clist_item);
  //GdkColor color = {0, 0, 0};
  //gtk_clist_set_foreground(GTK_CLIST(itemlistwidget), rowindex, &color);

  g_free(clist_item[COL_FILENAME]);
  g_free(clist_item[COL_RETRY]);
  g_free(clist_item[COL_REC]);
  g_free(clist_item[COL_CRC]);
  g_free(clist_item[COL_SAVE]);
  g_free(clist_item[COL_URL]);
  Set_clist_column__icon(rowindex, ItemCell::ITEM_READY);
  Set_clist_column__progress(rowindex, 0);
  gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
  itemcell->set_id(-1);
  item_manager->regist_item_back(itemcell);
}

/*
static gboolean Paste_option_default(GtkWidget *w, GtkWidget *additem_clist)
{
  console_item->Show_option_window();
  return TRUE;
}
*/

static void Paste_crc_set(const string& crc_strings)
{
  //strstream crcs;
  //crcs << crc_strings << ends;

  list<CRCList*> crc_list;
  unsigned int index = 0;
  while(index < crc_strings.size()) {
    string filename_entry;
    string crc_string;
    string size_string;
    unsigned int lf_pos, top_pos;
    if((lf_pos = crc_strings.find('\n', index)) == string::npos) {
      lf_pos = crc_strings.size();
    }
    if((top_pos = crc_strings.find_first_not_of(" \t", index)) != string::npos && top_pos < lf_pos) {
      index = lf_pos+1;
      string crc_entry_string = crc_strings.substr(top_pos, lf_pos);
      unsigned int start_pos, end_pos;
      if((end_pos = crc_entry_string.find_first_of(" \t")) == string::npos) continue;
      filename_entry = crc_entry_string.substr(0, end_pos);
      if((start_pos = crc_entry_string.find_first_not_of(" \t", end_pos)) == string::npos) continue;
      if((end_pos = crc_entry_string.find_first_of(" \t", start_pos)) == string::npos) end_pos = crc_entry_string.size();
      crc_string = crc_entry_string.substr(start_pos, end_pos-start_pos);
            
      CRCList *crc_entry = new CRCList(filename_entry, crc_string);
      crc_list.push_back(crc_entry);
    } else {
      index = lf_pos+1;
    }
  }
  const list<int>& download_list = item_manager->ret_id_list();
  for(list<int>::const_iterator id_itr = download_list.begin(); id_itr != download_list.end(); ++id_itr) {
    ItemCell *itemcell = item_manager->ret_itemaddr(*id_itr);
    for(list<CRCList*>::iterator crc_list_itr = crc_list.begin(); crc_list_itr != crc_list.end(); ++crc_list_itr) {
      if(itemcell->ret_Filename() == (*crc_list_itr)->ref_Filename()) {
	int rowindex;
	switch((*crc_list_itr)->ref_CRC_string().size()) {
	case 8:
	  itemcell->set_CRC(stoui((*crc_list_itr)->ref_CRC_string(), 16));
	  itemcell->set_CRC_Type(ItemCell::CRC_32);
	  rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
	  Set_clist_column__crc(rowindex, (*crc_list_itr)->ref_CRC_string());
	  crc_list.remove(*crc_list_itr);
	  delete *crc_list_itr;
	  break;
	case 4:
	  itemcell->set_CRC(stoui((*crc_list_itr)->ref_CRC_string(), 16));
	  itemcell->set_CRC_Type(ItemCell::CRC_16);
	  rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
	  Set_clist_column__crc(rowindex, (*crc_list_itr)->ref_CRC_string());
	  crc_list.remove(*crc_list_itr);
	  delete *crc_list_itr;
	  break;
	default:
	  cerr << "WARNING: unsupported CRC format" << endl;
	}
	break;
      }
    }
  }
  for(list<CRCList*>::iterator crc_list_itr = crc_list.begin(); crc_list_itr != crc_list.end(); ++crc_list_itr) {
    delete *crc_list_itr;
  }
}

//XΥåץܡɤURLŽդ
gboolean Paste_get_clipboard(GtkWidget *widget, gpointer data)
{
  static GdkAtom targets_atom = GDK_NONE;

  // ʸ "STRING" б륢ȥ
  if (targets_atom == GDK_NONE)
    targets_atom = gdk_atom_intern ("STRING", FALSE);

  // ץ饤ޥꥻ쥯 "STRING" åȤ׵
  gtk_selection_convert (widget, GDK_SELECTION_PRIMARY, targets_atom,
			 GDK_CURRENT_TIME);
  return TRUE;
}

// 쥯νͭԤǡ֤ȸƤФ륷ʥϥɥ
gboolean Paste_crc_selection_received(GtkWidget* widget, GtkSelectionData* selection_data, gpointer data)
{
  char *clipboard_contents;

  // ɤǧ뤿Υå
  if (selection_data->length < 0) {
#ifdef DEBUG
      cout << "Selection retrieval failed\n";
#endif
      return(TRUE);
  }
  // ԤǥǡȤǧ
  if (selection_data->type != GDK_SELECTION_TYPE_STRING) {
#ifdef DEBUG
    cout << "Selection \"TARGETS\" was not returned as atoms!\n";
#endif
    return(TRUE);
  }
       
  // äȥ
  clipboard_contents = (char *)selection_data->data;
  //cerr << "clipboard_contents:\n" << clipboard_contents << "\n" << flush;

  Paste_crc_set(clipboard_contents);
  return(TRUE);
}


// 쥯νͭԤǡ֤ȸƤФ륷ʥϥɥ
gboolean Paste_selection_received (GtkWidget *widget, GtkSelectionData *selection_data, gpointer data)
{
  char *clipboard_contents;

  // ɤǧ뤿Υå
  if (selection_data->length < 0) {
      return TRUE;
  }
  // ԤǥǡȤǧ
  if (selection_data->type != GDK_SELECTION_TYPE_STRING) {
    return TRUE;
  }       
  // äȥ
  clipboard_contents = (char*)selection_data->data;

  //gtk_clist_clear(GTK_CLIST(additem_clist));

  if(paste_window->add_url(clipboard_contents)) {
    paste_window->show();
  }

  return TRUE;
}

// URLŸƤŽդ
gboolean Paste_url_unfold_selection_received (GtkWidget *widget, GtkSelectionData *selection_data, gpointer data)
{
  char *clipboard_contents;

  // ɤǧ뤿Υå
  if (selection_data->length < 0) {
      return TRUE;
  }
  // ԤǥǡȤǧ
  if (selection_data->type != GDK_SELECTION_TYPE_STRING) {
    return TRUE;
  }
       
  // äȥ
  clipboard_contents = (char*)selection_data->data;

  if(paste_window->add_url_with_unfold(clipboard_contents)) {
    paste_window->show();
  }

  return TRUE;
}

void Setup_dnd(GtkWidget *toplevel, void (*Signal_Func)(GtkWidget  *w,
							GdkDragContext *context,
							int x,
							int y,
							GtkSelectionData *data,
							unsigned int info,
							unsigned int time))
{
  static GtkTargetEntry mime_types[] = {
    { "x-url/http", 0, MIME_URL },
    { "x-url/ftp", 0, MIME_URL },
    { "x-url/*", 0, MIME_URL },
    { "text/uri-list", 0, MIME_URL },
    { "_NETSCAPE_URL", 0, MIME_URL },
    { "text/plain", 0, MIME_TEXT_PLAIN },
    { "text/html", 0, MIME_TEXT_HTML }
  };

  static int n_mime_types = sizeof(mime_types)/sizeof(mime_types[0]);

  gtk_signal_connect(GTK_OBJECT(toplevel), "drag_data_received",
		     GTK_SIGNAL_FUNC(Signal_Func),
		     NULL);

  gtk_drag_dest_set(toplevel,
		    (GtkDestDefaults)(GTK_DEST_DEFAULT_MOTION |
				      GTK_DEST_DEFAULT_HIGHLIGHT |
				      GTK_DEST_DEFAULT_DROP),
		    mime_types, n_mime_types,
		    (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
}

void Create_edit_menu(GtkWidget *toplevel, GtkWidget *menu_bar, GtkAccelGroup *accel_group)
{
  GtkWidget *edit_menu, *edit_item, *paste_url_item, *paste_crc_item;
  GtkWidget *select_all_item;
  GtkWidget *clear_syslog;

  //create Edit menu
  edit_menu = gtk_menu_new();

  select_all_item = gtk_menu_item_new_with_label(_("select all items"));
  clear_syslog = gtk_menu_item_new_with_label(_("clear system log"));
  paste_url_item = gtk_menu_item_new_with_label(_("paste URL list"));
  GtkWidget *paste_url_unfold_item = gtk_menu_item_new_with_label(_("paste URL list with unfolding"));
  paste_crc_item = gtk_menu_item_new_with_label(_("paste CRC list"));
  
  
  gtk_menu_append(GTK_MENU(edit_menu), select_all_item);
  gtk_menu_append(GTK_MENU(edit_menu), clear_syslog);
  GtkWidget *separator_item = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(edit_menu), separator_item);
  gtk_widget_show(separator_item);

  gtk_menu_append(GTK_MENU(edit_menu), paste_url_item);
  gtk_menu_append(GTK_MENU(edit_menu), paste_url_unfold_item);
  gtk_menu_append(GTK_MENU(edit_menu), paste_crc_item);


  gtk_signal_connect(GTK_OBJECT(select_all_item), "activate",
		     GTK_SIGNAL_FUNC(Edit_select_all),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(clear_syslog), "activate",
		     GTK_SIGNAL_FUNC(Edit_clear_syslog),
		     NULL);
  // paste URL
  gtk_signal_connect(GTK_OBJECT(paste_url_item), "activate",
		     GTK_SIGNAL_FUNC(Paste_get_clipboard),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(paste_url_item), "selection_received",
		     GTK_SIGNAL_FUNC(Paste_selection_received),
		     NULL);
  // paste URL with unfolding
  gtk_signal_connect(GTK_OBJECT(paste_url_unfold_item), "activate",
		     GTK_SIGNAL_FUNC(Paste_get_clipboard),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(paste_url_unfold_item), "selection_received",
		     GTK_SIGNAL_FUNC(Paste_url_unfold_selection_received),
		     NULL);
  // paste CRC
  gtk_signal_connect(GTK_OBJECT(paste_crc_item), "activate",
		     GTK_SIGNAL_FUNC(Paste_get_clipboard),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(paste_crc_item), "selection_received",
		     GTK_SIGNAL_FUNC(Paste_crc_selection_received),
		     NULL);

  // accelerator
  gtk_widget_add_accelerator(select_all_item,
			     "activate",
			     accel_group,
			     SC_SELECT_ALL,
			     SCM_SELECT_ALL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(paste_url_item,
			     "activate",
			     accel_group,
			     SC_PASTEURL,
			     SCM_PASTEURL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(paste_url_unfold_item,
			     "activate",
			     accel_group,
			     SC_PASTEURLUNFOLD,
			     SCM_PASTEURLUNFOLD,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(paste_crc_item,
			     "activate",
			     accel_group,
			     SC_PASTECRC,
			     SCM_PASTECRC,
			     GTK_ACCEL_VISIBLE);


  gtk_widget_show(paste_url_item);
  gtk_widget_show(paste_url_unfold_item);
  gtk_widget_show(paste_crc_item);
  gtk_widget_show(select_all_item);
  gtk_widget_show(clear_syslog);

  edit_item = gtk_menu_item_new_with_label(_("Edit"));
  gtk_widget_show(edit_item);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_item), edit_menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), edit_item);
  // create paste window
  paste_window = new PasteWindow(GTK_WINDOW(toplevel));
}
