//
//  aria - yet another download tool
//  Copyright (C) 2000 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_file_open_url.cc,v 1.6 2001/03/08 08:49:55 tujikawa Exp $

#include "aria.h"
#include "ItemList.h"
#include "AppOption.h"
#include "ThreadManager.h"
#include "ItemCell.h"
#include "FileBrowser.h"

// external functions
extern void Set_sensitive__list_not_empty();
extern gboolean Hide_window(GtkWidget *window, gpointer unused);

// external variables
extern pthread_cond_t itemlistnotempty;
extern pthread_mutex_t itemlistlock;
extern unsigned int OK_connection_id, Cancel_connection_id;
extern GtkWidget *itemlistwidget;
extern ItemList *itemlist_global;
extern ItemCell *console_item;
extern AppOption *appoption;
extern ThreadManager *thread_manager;
extern GtkWidget *fs;
extern FileBrowser *cfilebrowser;
//
// URLեɤ߹
//
// URLե륻쥯󥦥ɥOKܥ򲡤Ȥν
void Open_url_file(const string& filename)
{
  pthread_mutex_lock(&itemlistlock);
  if(!itemlist_global->Read_URL_from_file(filename)) {
    pthread_mutex_unlock(&itemlistlock);
    console_item->Send_message_to_gui(_("Error occurd while reading URL file"), MSG_SYS_ERROR);
  } else {
    bool crc_flag = false;
    
    struct stat file_stat;
    string crc_filename = filename+".crc";
    if(stat(crc_filename.c_str(), &file_stat) == 0 && S_ISREG(file_stat.st_mode)) {
      crc_flag = true;
    } else {
      unsigned int ext_pos = filename.rfind('.');
      if(ext_pos != string::npos) {
	crc_filename = filename.substr(0, ext_pos)+".crc";
	if(stat(crc_filename.c_str(), &file_stat) == 0 && S_ISREG(file_stat.st_mode)) {
	  crc_flag = true;
	}
      }
    }
    if(crc_flag) {
      itemlist_global->Read_CRC_from_file(crc_filename);
    }
    if(appoption->Whether_use_automatic_start() ||
       !thread_manager->Is_all_threads_sleeping()) {
      pthread_cond_broadcast(&itemlistnotempty);
    }
    pthread_mutex_unlock(&itemlistlock);
    console_item->Send_message_to_gui(_("URL list opened"), MSG_SYS_INFO);
  }
  if(GTK_CLIST(itemlistwidget)->rows > 0) {
      Set_sensitive__list_not_empty();
  }
}

static gboolean File_ok_open_URL_list(GtkWidget *w, GtkWidget *fs)
{
  const char *filename;

  cfilebrowser->hide();
  if((filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs))) != NULL) {
    Open_url_file(filename);
  }
  return TRUE;
}

gboolean File_open_URL_list(GtkWidget *w, gpointer data)
{
  cfilebrowser->setup(_("Open URL list"),
		      File_ok_open_URL_list);
  cfilebrowser->show();  
  return(TRUE);
}
