//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: main.cc,v 1.35 2001/03/22 08:42:18 tujikawa Exp $

#include <stdlib.h>
#include <iomanip.h>
#include <fstream.h>
#include <unistd.h>
#include "getopt.h"
#include <stdlib.h>
#include <pthread.h>
#include <sys/utsname.h>
#include "ProxyList.h"
#include "ItemCell.h"
#include "ItemList.h"
#include "ItemManager.h"
#include "AppOption.h"
#include "ItemOption.h"
#include "ThreadManager.h"
#include "FileBrowser.h"
#include "ServerTemplateList.h"
#include "CommandList.h"
#include "CtrlSocket.h"
#include <iostream>
#include <string>
#include "aria.h"
using namespace std;

extern GtkWidget *GUI_main();
extern void *Download_thread_main();
extern void *Set_sensitive__list_not_empty();
extern void Toolbar_set_sensitive__thread();
extern void Set_file_selection_default_path(const string& selected_dir);
extern FileBrowser *cfilebrowser;
extern GtkWidget *itemlistwidget, *text, *console_text;
extern bool Create_new_item(const string& url, const string& dir = "", const string& referer = "");

// global variables
ThreadManager *thread_manager;
AppOption *appoption;
ItemList *itemlist_global;
ItemCell *console_item;
ItemOption *itemoption;
ItemManager *item_manager;
ItemManager *item_manager_paste;
ProxyList *http_proxy_list = NULL;
UseragentList *useragent_list = NULL;
pthread_mutex_t itemlistlock;
pthread_mutex_t guiupdate_lock;
pthread_cond_t itemlistnotempty;
int pipetogui[2];
ServerTemplateList svt_list;
CommandList command_list;
CtrlSocket ctrlsock;
string machine_info;
unsigned int thread_limit;

void Show_usage()
{
  cerr << "Usage: aria [OPTION]...\n";
  cerr << "Yet another download tool\n\n";
  
  // options listing
  cerr << "Options:\n";
  cerr << "  -g, --get URL                    URL to get\n";
  cerr << "  -d, --dir SAVEDIR                save directory\n";
  cerr << "  -r, --ref REFERER                referer\n";
  cerr << "  -u, --url-list FILE              open URL list\n";
  cerr << "  -c, --crc-list FILE              open CRC list\n";
  cerr << "  -v, --version                    print version information\n";
  cerr << "  -h, --help                       print this help\n";
  cerr << '\n';
  cerr << "  --thread-limit-override NUM      if you want more simultaneous downloads..." << '\n';
  cerr << "\nControl Options:\n following options are effective only when another aria is already running\n\n";
  cerr << "  -s, --start                      start all downloads\n";
  cerr << "  -p, --stop                       stop all downloads\n";
  cerr << "  -a, --again                      re-download all items\n";
  cerr << "  -q, --quit                       quit Aria(with no conformation)\n";
  cerr << "  --clear-complete                 clear downloaded items\n";
  cerr << "  --ls-complete                    show URLs downloaded\n";
  cerr << "  --ls-ready                       show URLs waiting in queue\n";
  cerr << "  --ls-error                       show URLs in error\n";
  cerr << "  --ls-run                         show URLs now downloading\n";
  cerr << "  --ls-stop                        show URLs stopped\n";
  cerr << "  --ls-lock                        show URLs locked\n";
  cerr << "  --ls-all                         show all URLs in queue\n";
}

void Show_version()
{
  cerr << "Aria (version " << VERSION << "; " << D_MACHINE_COMPILED << ")" << '\n';
  cerr << "compiled date: " << D_DATE_COMPILED << '\n';
  cerr << "\nCopyright (C) 2000, 2001 Tatsuhiro Tsujikawa\n";

  //cerr << "\nAny suggestions or bug reports: tujikawa@pop16.odn.ne.jp" << endl;
}

void create_default_filter_nodown_target_list(list<string>& filter_target_list)
{
  filter_target_list.push_back(".tgz");
  filter_target_list.push_back(".tar.gz");
  filter_target_list.push_back(".tar.bz2");
  filter_target_list.push_back(".deb");
  filter_target_list.push_back(".rpm");
  filter_target_list.push_back(".zip");
  filter_target_list.push_back(".exe");
  filter_target_list.push_back(".cgi");
  filter_target_list.push_back(".bin");
  /*
  filter_target_list.push_back(".html");
  filter_target_list.push_back(".htm");
  filter_target_list.push_back(".jpg");
  filter_target_list.push_back(".jpeg");
  filter_target_list.push_back(".png");
  filter_target_list.push_back(".gif");
  filter_target_list.push_back(".php");
  filter_target_list.push_back(".php3");
  filter_target_list.push_back(".asp");
  */
}

void create_default_parse_target_list(list<string>& parse_target_list)
{
  parse_target_list.push_back(".html");
  parse_target_list.push_back(".htm");
  parse_target_list.push_back(".shtml");
  parse_target_list.push_back(".php3");
}

// å
void ctrlsock_getmsg(void *w, int ctrl_fd, GdkInputCondition dummy_cond)
{
  ctrlsock.Read_message();
}

void receive_list(int fd)
{
  cout << "Status (Ready/Error/Downloading/Stop/Complete/Lock)\n";
  cout << "| Split?(Yes/No/split Parts)\n";
  cout << "|/\n";
  cout << "++-=============================================================================\n";
  int count = 0;
  while(1) {
    char *data;
    int size = ctrlsock.Recv_command(fd, (void **)&data);
    if(size < 0) {
      return;
    } else if(size == 0) {
      break;
    } else {
      cout << data << endl;
      delete [] data;
    }
    ctrlsock.Send_command(fd, CmdPacketHeader::CMD_PING);
    ++count;
  }
  cout << '\n' << count << " URLs found\n";
}

void show_no_remote_warning()
{
  cerr << "Another Aria is not running. This option is safely ignored." << endl;
}

int main(int argc, char **argv)
{
  int option_index;
  int c;
  string filename;
  string crcfilename;
  string svtfilename;
  static struct option long_options[] = 
  {
    { "get", required_argument, NULL, 'g' },
    { "dir", required_argument, NULL, 'd' },
    { "ref", required_argument, NULL, 'r' },
    { "url-list", required_argument, NULL, 'u' },
    { "crc-list", required_argument, NULL, 'c' },
    { "help", no_argument, NULL, 'h' },
    { "version", no_argument, NULL, 'v' },
    { "start", no_argument, NULL, 's' },
    { "stop", no_argument, NULL, 'p' },
    { "again", no_argument, NULL, 'a' },
    { "quit", no_argument, NULL, 'q' },
    { "clear-complete", no_argument, NULL, 'm' },
    { "ls-complete", no_argument, NULL, 't' },
    { "ls-error", no_argument, NULL, 'e' },
    { "ls-ready", no_argument, NULL, 'w' },
    { "ls-stop", no_argument, NULL, 'b' },
    { "ls-lock", no_argument, NULL, 'k' },
    { "ls-run", no_argument, NULL, 'x' },
    { "ls-all", no_argument, NULL, 'y' },
    { "thread-limit-override", required_argument, NULL, 'n' },
    { 0, 0, 0, 0}
  };
  // open pipe
  pipe(pipetogui);

  // process gtk-specific arguments
  g_thread_init(NULL);
# ifdef ENABLE_NLS
  bindtextdomain(PACKAGE, LOCALEDIR);
  textdomain(PACKAGE);
# endif
  gtk_set_locale();
  gtk_init(&argc, &argv);

  // read configuration file: rc.aria
  string home_dir = g_get_home_dir();
  string rc_dir =  home_dir+"/.aria/";
  //Make_configuration_directory(rc_dir);
  string rc_filename = rc_dir+"rc.aria";
  gtk_rc_parse(rc_filename.c_str());
  string url;
  string dir;
  string ref;

  thread_limit = THREADLIMIT;//default thread limit
  while((c = getopt_long(argc, argv, "g:d:r:u:c:hvspaq", long_options, &option_index)) != EOF)
    {
      switch(c)
	{
	case 'u':
	  if(!filename.empty()) {
	    cerr << "ERROR: don't specify multitple URL list file" << endl;
	    Show_usage();
	    exit(1);
	  }
	  filename = optarg;
	  if(ctrlsock.Is_remote_running()) {
	    if(filename.at(0) != '/') {
	      char *gtmp = g_get_current_dir();
	      string tmp = gtmp;
	      filename = tmp+'/'+filename;
	      g_free(gtmp);

	    }
	    ctrlsock.Send_command(CmdPacketHeader::CMD_URLLIST, (void *)filename.c_str(), filename.size()+1);
	  }
	  break;
	case 'c':
	  if(!crcfilename.empty()) {
	    cerr << "ERROR: don't specify multitple CRC list file" << endl;
	    Show_usage();
	    exit(1);
	  }
	  crcfilename = optarg;
	  if(ctrlsock.Is_remote_running()) {
	    if(filename.at(0) != '/') {
	      char *gtmp = g_get_current_dir();
	      string tmp = gtmp;
	      filename = tmp+'/'+filename;
	      g_free(gtmp);
	    }
	    ctrlsock.Send_command(CmdPacketHeader::CMD_CRCLIST, (void *)filename.c_str(), filename.size()+1);
	  }
	  break;
	case 's':
	  if(ctrlsock.Is_remote_running()) {
	    ctrlsock.Send_command(CmdPacketHeader::CMD_START);
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'p':
	  if(ctrlsock.Is_remote_running()) {
	    ctrlsock.Send_command(CmdPacketHeader::CMD_STOP);
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'a':
	  if(ctrlsock.Is_remote_running()) {
	    ctrlsock.Send_command(CmdPacketHeader::CMD_AGAIN);
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'q':
	  if(ctrlsock.Is_remote_running()) {
	    ctrlsock.Send_command(CmdPacketHeader::CMD_QUIT);
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'm':
	  if(ctrlsock.Is_remote_running()) {
	    ctrlsock.Send_command(CmdPacketHeader::CMD_DELCOMPLETE);
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 't':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSCOMP);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'e':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSFAILED);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'w':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSREADY);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'b':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSSTOP);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'x':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSGO);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'k':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSLOCK);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'y':
	  if(ctrlsock.Is_remote_running()) {
	    int fd;
	    if((fd = ctrlsock.connect_to()) > 0) {
	      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_LSALL);
	      receive_list(fd);
	      close(fd);
	    }
	  } else {
	    show_no_remote_warning();
	  }
	  break;
	case 'n': {
	  int new_limit = stoi(optarg);
	  if(new_limit > 0 && new_limit <= MAXTHREAD) {
	    thread_limit = new_limit;
	  } else {
	    cerr << "thread limit exceeded or invalid. valid range is 1 to " << MAXTHREAD << ".\n";
	    cerr << "Abort." << endl;
	    exit(1);
	  }
	  break;
	}
	case 'h':
	  Show_usage();
	  exit(0);
	case 'v':
	  Show_version();
	  exit(0);
	case 'g':
	  url = optarg;
	  break;
	case 'd':
	  dir = optarg;
	  break;
	case 'r':
	  ref = optarg;
	  break;
	case '?':
	  exit(0);
	  break;
	}
    }
  if(ctrlsock.Is_remote_running()) {
    if(url.size()) {
      // connect to remote Aria
      int fd = ctrlsock.connect_to();
      // send URL
      ctrlsock.Send_command(fd, CmdPacketHeader::CMD_URL, (void *)url.c_str(), url.size()+1);
      if(dir.size()) {
	// send SAVEDIR
	try {
	  if(dir.at(0) != '/') {
	    gchar *temp = g_get_current_dir();
	    dir = get_storedir(temp, dir);
	    g_free(temp);
	  }
	  ctrlsock.Send_command(fd, CmdPacketHeader::CMD_URL, (void *)dir.c_str(), dir.size()+1);
	} catch (int err) {
	  cerr << "ERROR: invalid save dir" << endl;
	  ctrlsock.Send_command(fd, CmdPacketHeader::CMD_URL);
	}
      } else {
	// if no savedir specified, then send ""
	ctrlsock.Send_command(fd, CmdPacketHeader::CMD_URL);
      }
      if(ref.size()) {
	// send REFERER
	ctrlsock.Send_command(fd, CmdPacketHeader::CMD_URL, (void *)ref.c_str(), ref.size()+1);
      } else {
	// if no referer specified, then send ""
	ctrlsock.Send_command(fd, CmdPacketHeader::CMD_URL);
      }
      // close connection
      close(fd);
    }
    exit(0);
  } else {
    // add input watch handler
    gdk_input_add(ctrlsock.ret_ctrl_fd(), GDK_INPUT_READ, ctrlsock_getmsg, NULL);
  }
  // get uname
  struct utsname *uname_st_ptr = new struct utsname;  
  if(uname(uname_st_ptr) < 0) {
    machine_info = D_MACHINE_COMPILED;
  } else {
    char *temp = g_strconcat(uname_st_ptr->sysname, " ", uname_st_ptr->release, " ", uname_st_ptr->machine, NULL);
    machine_info = temp;
    g_free(temp);
  }
  delete uname_st_ptr;

  pthread_mutex_init(&itemlistlock, NULL);
  pthread_mutex_init(&guiupdate_lock, NULL);
  pthread_cond_init(&itemlistnotempty, NULL);

  srandom(time(NULL));
  itemlist_global = new ItemList();
  GtkWidget *app_window = GUI_main();
  item_manager = new ItemManager();
  item_manager_paste = new ItemManager();

  itemoption = new ItemOption(app_window);


  console_item = new ItemCell();
  console_item->set_logging(false);
  // read server config file
  console_item->Send_message_to_gui(_("Reading Server Template file.."), MSG_SYS_INFO);
  if(svtfilename.empty()) svtfilename = itemlist_global->ret_file_server_settings();
  if(svt_list.Read_config_file(svtfilename)) {
    console_item->Send_message_to_gui(_("Server Template status : good"), MSG_SYS_INFO);
  } else {
    console_item->Send_message_to_gui(_("Server Template status : error"), MSG_SYS_ERROR);
  }
  // read command list file
  console_item->Send_message_to_gui(_("Reading Command list file.."), MSG_SYS_INFO);

  if(command_list.Read_from_file(itemlist_global->ret_file_command_list())) {
    console_item->Send_message_to_gui(_("Command list status : good"), MSG_SYS_INFO);
  } else {
    console_item->Send_message_to_gui(_("Command list status : error"), MSG_SYS_ERROR);
  }

  useragent_list = new UseragentList();
  //char *original_useragent = g_strconcat("Aria/", VERSION, " (", D_MACHINE_COMPILED, ")", NULL);
  
  char *original_useragent = g_strconcat("Aria/", VERSION, " (", machine_info.c_str(), ")", NULL);
  useragent_list->add(original_useragent);
  g_free(original_useragent);
  useragent_list->Read_useragent_list(itemlist_global->ret_file_useragent_list());
  console_item->Send_message_to_gui(_("Reading Default item config file..."), MSG_SYS_INFO);

  if(!itemlist_global->Restore_saved_default_item_settings()) {
    console_item->Send_message_to_gui(_("Default item config status: error >> using defualt config"), MSG_SYS_ERROR);
    char *current_dir = g_get_current_dir();
    Proxyserver proxy;
    Userdata user(DEFAULT_USER, DEFAULT_PASSWORD);
    Userdata http_proxy_user(DEFAULT_HTTP_PROXY_USER, DEFAULT_HTTP_PROXY_PASSWORD);
    list<string> filter_down_target_list;
    list<string> filter_nodown_target_list;
    create_default_filter_nodown_target_list(filter_nodown_target_list);
    list<string> parse_target_list;
    create_default_parse_target_list(parse_target_list);
    list<string> ign_domain_list;
    list<string> ftp_filter_target_list;
    console_item->ret_Options_opt().Change_Values(DEFAULT_USE_AUTHENTICATION,
						  user,
						  current_dir,
						  DEFAULT_HTTP_VERSION,
						  DEFAULT_REFERER_INDEX,
						  DEFAULT_REFERER_STRING,
						  useragent_list->ret_vector().front(),
						  DEFAULT_RANDOM_USERAGENT,
						  DEFAULT_USE_HTTP_PROXY,
						  DEFAULT_USE_HTTP_CACHE,
						  DEFAULT_USE_HTTP_PROXY_AUTHENTICATION,
						  http_proxy_user,
						  proxy,
						  DEFAULT_TIMEDOUT,
						  DEFAULT_DIVIDE,
						  DEFAULT_ROLLBACK_BYTES,
						  DEFAULT_RECURSE_COUNT,
						  DEFAULT_WITH_HOSTNAME_DIR,
						  DEFAULT_ABS2REL,
						  DEFAULT_FORCE_CONVERT,
						  DEFAULT_DEL_COMMENT,
						  DEFAULT_DEL_JAVASCRIPT,
						  DEFAULT_NO_OTHER_HOST,
						  DEFAULT_NO_ASCEND,
						  DEFAULT_RELATIVE_ONLY,
						  DEFAULT_REFERER_OVERRIDE,
						  DEFAULT_FOLLOW_FTP_LINK,
						  DEFAULT_CONVERT_TILDE,
						  DEFAULT_TAG_HREF,
						  DEFAULT_TAG_SRC,
						  DEFAULT_TAG_BACKGROUND,
						  DEFAULT_TAG_CODE,
						  DEFAULT_USE_DOWN_FILTER,
						  filter_down_target_list,
						  filter_nodown_target_list,
						  parse_target_list,
						  ign_domain_list,
						  DEFAULT_FTP_MODE,
						  DEFAULT_FTP_RET_MODE,
						  DEFAULT_FTP_NOSEND_QUIT,
						  DEFAULT_FTP_RECURSE_COUNT,
						  DEFAULT_FTP_USE_FILTER,
						  DEFAULT_FTP_ALLOW_CRAWL_SUBDIR,
						  DEFAULT_FTP_NO_ASCEND,
						  DEFAULT_FTP_GET_SYMLINK_AS_REALFILE,
						  ftp_filter_target_list,
						  DEFAULT_IFDEL,
						  DEFAULT_RETRY,
						  DEFAULT_RETRY_REPEAT,
						  DEFAULT_FORCE_RETRY_404,
						  DEFAULT_FORCE_RETRY_503,
						  DEFAULT_IFCRC,
						  DEFAULT_NO_CRC_CHECKING,
						  DEFAULT_IGNORE_CRC_ERROR,
						  DEFAULT_COOKIE_DELETE_ON_RESTART,
						  DEFAULT_COOKIE_NOSEND,
						  DEFAULT_DOWNM_TYPE,
						  DEFAULT_SPEED_LIMIT);
    g_free(current_dir);
    //console_item->ret_Options_opt().activate_filter_down_target_list(filter_down_target_list);
    console_item->ret_Options_opt().activate_filter_nodown_target_list(filter_nodown_target_list);
    console_item->ret_Options_opt().activate_parse_target_list(parse_target_list);
  } else {
    console_item->Send_message_to_gui(_("Default item config status: good"), MSG_SYS_INFO);
  }
  console_item->set_Status(ItemCell::ITEM_CONSOLE);
  http_proxy_list = new ProxyList();
  http_proxy_list->Read_proxy_list(itemlist_global->ret_file_http_proxy_list());
  // application settings
  appoption = new AppOption(app_window);
  pthread_mutex_lock(&itemlistlock);
  console_item->Send_message_to_gui(_("Reading Application config file.."), MSG_SYS_INFO);
  if(!itemlist_global->Restore_saved_app_settings()) {
    console_item->Send_message_to_gui(_("Application config status: error >> using defualt config"), MSG_SYS_ERROR);
    list<string> ignore_list;
    ignore_list.push_back("html");
    ignore_list.push_back("cgi");
    ignore_list.push_back("pl");
    list<string> dummy;
    TimerData timerdata;
    appoption->Set_Option_Values(1, true, false, false, ignore_list, false, 10, false, "", false, timerdata, true, true, true, false, true, DEFAULT_ICONSET, dummy, dummy);
  } else {
    console_item->Send_message_to_gui(_("Application config status: good"), MSG_SYS_INFO);
  }
  thread_manager = new ThreadManager(appoption->ret_Maxthread());
  pthread_mutex_unlock(&itemlistlock);

  //set http_proxy_list and useragent_list
  itemoption->set_http_proxy_list(http_proxy_list);
  itemoption->set_useragent_list(useragent_list);

  if(!filename.empty()) {
    if(!itemlist_global->Read_URL_from_file(filename)) {
      //cerr << "ERROR:URL file read error" << endl;
      console_item->Send_message_to_gui(_("URL file read error"), MSG_SYS_ERROR);
    } else {
      console_item->Send_message_to_gui(_("URL file opend"), MSG_SYS_INFO);
    }
  } else {
    if(itemlist_global->Restore_saved_list()) {
      console_item->Send_message_to_gui(_("Saved file opened"), MSG_SYS_INFO);
    }
  }
  if(GTK_CLIST(itemlistwidget)->rows > 0) {
    Set_sensitive__list_not_empty();
  }

  // this directory is used as the default directory of file selection widgets
  cfilebrowser->set_default_path(console_item->ret_Options_opt().ret_Store_Dir());

  /// crc file
  if(!crcfilename.empty()) {
    if(!itemlist_global->Read_CRC_from_file(crcfilename)) {
      console_item->Send_message_to_gui(_("CRC file read error"), MSG_SYS_ERROR);
    } else {
      console_item->Send_message_to_gui(_("CRC file opened"), MSG_SYS_INFO);
    }
  }

  string version_info;
  version_info = version_info+"Aria (version "+VERSION+"; "+machine_info+") Ready";
  console_item->Send_message_to_gui(version_info.c_str(), MSG_SYS_INFO);

  Toolbar_set_sensitive__thread();

  /*
  list<char *>::iterator itr = url_arg.begin();
  if(url_arg.size() == 1) {
    Create_new_item(*itr);
  } else if(url_arg.size() == 2) {
    try {
      string url = *itr;
      ++itr;
      string fix;
      if(*(*itr) != '/') {
	gchar *temp;
	fix = (temp = g_get_current_dir());
	fix = get_storedir(fix, *itr);
	g_free(temp);
      } else {
	fix = *itr;
      }
      Create_new_item(url, fix);
    } catch (int err) {
      // nothing to do here
    }
  }
  */
  if(url.size()) {
    if(dir.size() && dir.at(0) != '/') {
      gchar *temp = g_get_current_dir();
      dir = get_storedir(temp, dir);
      g_free(temp);
    }
    Create_new_item(url, dir, ref);
  }

  /*
  for(list<char *>::iterator itr = url_arg.begin(); itr != url_arg.end(); ++itr) {
    char *eq_ptr;
    if((eq_ptr = strchr(*itr, '=')) == NULL) {
      Create_new_item(*itr);
    } else {
      string dir = eq_ptr+1;
      *(eq_ptr) = '\0'; 
      Create_new_item(*itr, dir);
    }
  }
  */
  //url_arg.clear();

  if(appoption->Whether_use_automatic_start()) {
    pthread_mutex_lock(&itemlistlock);
    pthread_cond_broadcast(&itemlistnotempty);
    pthread_mutex_unlock(&itemlistlock);
  }

  nice(20);
  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
  
  exit(0);
}
